#ifndef DIG_FILE_SNARK_PRJ_H
#define DIG_FILE_SNARK_PRJ_H

#include <string.h>

//#include "DIGFile.h"
#include <DIGFile/DIGFile.h>

enum DetectorTypeEnum {
	//DT_UNSET,
	DT_TANGENT,
	DT_ARC,
	DT_UNIFORM,
	DT_VARIABLE,
};

enum ProjTypeEnum {
	//PT_UNSET,
	PT_STRIP,
	PT_LINE,
};

enum GeometryTypeEnum {
	//GT_UNSET,
	GT_DIVERGENT,
	GT_PARALLEL,
};

class ProjGeometry
{
public:
  //geometry
	GeometryTypeEnum  GeometryType;
	DetectorTypeEnum  DetectorType;

  // Divergent
	double            Radius; 
	double            SrcDetDistance;

  // Paralel
	ProjTypeEnum      ProjType;

public:
  ProjGeometry(double, double, DetectorTypeEnum);
	ProjGeometry(DetectorTypeEnum, ProjTypeEnum);
};


// Noise

class ProjNoise
{
public:
 
  // quantum
	bool		IsQuan;
	double  QuantumMean;
	double  QuantumCalibration;
	int     QuantumGantry;

  // scatter
  bool    IsScat;
	double  ScatterPeak;
	double  ScatterWidth;

  // additive
  bool    IsAdd;
	double  AdditiveMean;
	double  AdditiveStdDev;

  // multiplicative
  bool    IsMult;
	double  MultiplicativeMean;
	double  MultiplicativeStdDev;


  ProjNoise();

	void SetQuantum(double, double, int);
	void SetScatter(double, double);
	void SetAdditive(double, double);
	void SetMultiplicative(double, double);


  /*
	int GetQuantum(double&, double&, double&);
	int GetScatter(double&, double&);
	int GetAdditive(double&, double&);
	int GetMultiplicative(double&, double&);
  */
};

// spectrum
class ProjSpectrum
{
public:
	unsigned int  NumberOfEnergies;
	int*          energy;
	double*       ratio;
	double*       background;

  ProjSpectrum(unsigned int eNum);
  int SetEnergy(int, int, double, double);

  /*
  int SetSpectrum(int* e, double* r, double* bg);


	int GetNumberOfEnergies(unsigned int* eNum);
	int GetEnergy(int, int*, double*, double*);
  */
};

///////////////////////////////////////////////////////////////////////////////
// DIGFileSnarkPrj main header
///////////////////////////////////////////////////////////////////////////////

class ProjFileMH
{
public:
  DIGString     Title;
  unsigned int  Dimensions;
  double        Sampling;
  DIGString     Comment;
};

///////////////////////////////////////////////////////////////////////////////
// DIGFileSnarkPrj application header
///////////////////////////////////////////////////////////////////////////////

struct ProjFileAH
{
public:
  DIGString Comment;

  // geometry

  GeometryTypeEnum    GeometryType;
  DetectorTypeEnum    DetectorType;

  double              Radius;
  double              SrcDetDistance;

  ProjTypeEnum        ProjType;

  // spectrum

  unsigned int  NoOfEnergies;

  int*          Energy;
  double*       Ratio; 
  double*       Background;

  // noise

  bool    QuantumFlag;
  double  QuantumM; 
  double  QuantumCM;
  int     QuantumGA;

  bool    ScatterFlag;
  double  ScatterP;
  double  ScatterW;

  bool    AdditiveFlag;
  double  AdditiveM;
  double  AdditiveSD;

  bool    MultiplicativeFlag;
  double  MultiplicativeM;
  double  MultiplicativeSD;
};

///////////////////////////////////////////////////////////////////////////////
// DIGFileSnarkPrj projection header
///////////////////////////////////////////////////////////////////////////////

class ProjFilePrjH
{
public:
  unsigned int  ProjNo;
  double        Angle;
  DIGString     Comment;
};



class DIGFileSnarkProj: protected DIGFile {

protected:
  static const char* TypeStr;
  static const char* SchemaStr;

  static bool XMLStringsInitialized;

	static const XMLCh* AppHeaderXMLStr;
	static const XMLCh* GeomHeaderXMLStr;
	static const XMLCh* DivergentXMLStr;
	static const XMLCh* RadiusXMLStr;
	static const XMLCh* SourceDetXMLStr;
	static const XMLCh* DetTypeXMLStr;
	static const XMLCh* ParallelXMLStr;
	static const XMLCh* ProjTypeXMLStr;
	static const XMLCh* NoiseHeaderXMLStr;
	static const XMLCh* QuantumXMLStr;
	static const XMLCh* MeanXMLStr;
	static const XMLCh* CalibMeasXMLStr;
	static const XMLCh* GantryXMLStr;
	static const XMLCh* ScatterXMLStr;
	static const XMLCh* PeakXMLStr;
	static const XMLCh* WidthXMLStr;
	static const XMLCh* AdditiveXMLStr;
	static const XMLCh* StdDevXMLStr;
	static const XMLCh* MultipXMLStr;
	static const XMLCh* SpectrumHeaderXMLStr;
	static const XMLCh* NumOfEnerXMLStr;
	static const XMLCh* EnergyLevelXMLStr;
	static const XMLCh* EnergyXMLStr;
	static const XMLCh* RatioXMLStr;
	static const XMLCh* BackgroundXMLStr;
	
  DIGString      AppComment;
  ProjSpectrum*  Spectrum;
  ProjGeometry*  Geometry;
  ProjNoise      Noise;

public:

	DIGFileSnarkProj();
	~DIGFileSnarkProj();

  //
  // Writing
  //

  int Open(const char* pFileName, ProjFileMH* pMainHeader);
  int Open(
    const char*  pFileName,     // File Name
    const char*  pTitle,        // Title
    unsigned int pNumberOfRays, // USRAYS
    double       pSampling,     // PINC
    const char*  pComment       // Comment
  );

  // updating main header after opening file to write

  int SetTitle(const char* pTitle) {
    return DIGFile::SetTitle(pTitle);
  }

  int SetComment(const char* pComment) {
    return DIGFile::SetComment(pComment);
  }

  // writing application header

  int SetAppHeader(ProjFileAH* pAppHeader);

  // comment
  int SetAppComment(const char* pComment);

  // geometry
  int SetGeometry(double r, double d, DetectorTypeEnum t);  // divergent
  int SetGeometry(DetectorTypeEnum t, ProjTypeEnum p);      // parallel

  // spectrum
  int SetSpectrum(unsigned int eNum, int* e, double* r, double* bg);

  // noise
	int SetNoiseQuantum(double, double, int);
	int SetNoiseScatter(double, double);
	int SetNoiseAdditive(double, double);
	int SetNoiseMultiplicative(double, double);

  // Appending Projection

  int AppendProj(ProjFilePrjH* pProjHeader, const double* pData);
  int AppendProj(
    unsigned int  pProjNo,
    double        pAngle,
    const char*   pComment,
    const double* pData
  );

  //
  // Reading
  //

  int Open(const char* pFileName);

  // Reading file parameters

  int GetNoOfRays(unsigned int* pNoOfRays) {
    return DIGFile::GetArrayNoOfItems(pNoOfRays);
  }

  int GetNoOfProjs(unsigned int* pProjections) {
    return DIGFile::GetNoOfArraySets(pProjections);
  }

  // file navigation

  int SelectProj(unsigned int No) {
    int ret;

    if((ret = SelectArraySet(No)) != 0) {
      return ret;
    }

    return SelectArray(0);
  }

  // Reading main header

  int GetMainHeader(ProjFileMH* pMainHeader);

  int GetTitle(const char** pTitle) {
    return DIGFile::GetTitle(pTitle);
  }

  int GetEndian(DIGEndian* pEndian) {
    return DIGFile::GetEndian(pEndian);
  }

  int GetDimensions(unsigned int* pDimensions) {
    const DIGDimensions* Dimensions;
    int ret;

    if((ret = DIGFile::GetDimensions(&Dimensions)) != 0) {
      return ret;
    }

    *pDimensions = Dimensions->x;

    return 0;
  }

  int GetSampling(double* pSampling) {
    const DIGSampling* Sampling;
    int ret;

    if((ret = DIGFile::GetSamplingX(&Sampling)) != 0) {
      return ret;
    }

    *pSampling = Sampling->x;

    return 0;
  }

  int GetComment(const char** pComment) {
    return DIGFile::GetComment(pComment);
  }

  // Reading application header 

  int GetAppHeader(ProjFileAH* pAppHeader);

  // comment
  int GetAppComment(const char** pComment);

  // geometry
  int GetGeometryType(GeometryTypeEnum* gt);
  int GetGeometry(double* r, double* d, DetectorTypeEnum* t); // divergent
  int GetGeometry(DetectorTypeEnum* t, ProjTypeEnum* p);      // parallel

  // spectrum
  int GetNoOfEnergies(unsigned int* NumbOfEnergies);
  int GetSpectrum(unsigned int i, int* e, double* r, double* bg);

  // noise
  int HasNoiseQuantum(bool* a);
  int GetNoiseQuantum(double* m, double* cm, int* ga);
  int HasNoiseScatter(bool* a);
  int GetNoiseScatter(double* p, double* w);
  int HasNoiseAdditive(bool* a);
  int GetNoiseAdditive(double* m, double* sd);
  int HasNoiseMultiplicative(bool* a);
  int GetNoiseMultiplicative(double* m, double* sd);


  // Reading projection header

  int GetProjHeader(ProjFilePrjH* pProjHeader);

  int GetProjNo(unsigned int* pProjNo) 
  {
    int ret;
    const char* ProjNoStr;

    if((ret = DIGFile::GetArraySetTitle(&ProjNoStr)) != 0) {
      return ret;
    }
 
    *pProjNo = atoi(ProjNoStr);

    return 0;
  }

  int GetProjAngle(double* pAngle) {
    int ret;
    const char* Parameters;

    if((ret = DIGFile::GetArraySetParameters(&Parameters)) != 0) {
      return ret;
    }
 
    *pAngle = atof(Parameters);

    return 0;
  }

  int GetProjComment(const char** pComments) {
    return DIGFile::GetArraySetComment(pComments);
  }

  // Reading projection data

  int GetProjData(void* pArrayData) {
    return DIGFile::GetArrayData(pArrayData);
  }

  int Close() {
    int ret = DIGFile::Close();

    if(Geometry != NULL) {
	    delete Geometry;
      Geometry = NULL;
    }

    if(Spectrum != NULL) {
	    delete Spectrum;
      Spectrum = NULL;
    }

    return ret; 
  }

protected:

	int ParseApplicationHeader();
	int CreateApplicationHeader();
};

#endif // SNARK_PRJ_FILE_H
