#include <cstdio>
#include <string>

//#include "DIGFileSnarkProj.h"
#include <DIGFileSnark/DIGFileSnarkProj.h>

#define DIGFILESNARKPROJ_DEBUG_LEVEL 0

static char DIGFileSnarkPrjEmptyStr[1] = "";

// Geometry

// divergent
ProjGeometry::ProjGeometry(double pRadius, double pSrcDetDistance, DetectorTypeEnum pDetectorType)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  printf("ProjGeometry::ProjGeometry\n");
#endif

	Radius = pRadius;
	SrcDetDistance = pSrcDetDistance;
	DetectorType = pDetectorType;
	GeometryType = GT_DIVERGENT;
}

// parallel
ProjGeometry::ProjGeometry(DetectorTypeEnum pDetectorType, ProjTypeEnum pProjType)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  printf("ProjGeometry::ProjGeometry\n");
#endif

	DetectorType = pDetectorType;
	ProjType = pProjType;
	GeometryType = GT_PARALLEL;
}


// Energy Spectrum

ProjSpectrum::ProjSpectrum(unsigned int eNum)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  printf("ProjSpectrum::ProjSpectrum\n");
#endif

	NumberOfEnergies = eNum;
	energy = new int[eNum];
	ratio = new double[eNum];
  background = new double[eNum];
}


int ProjSpectrum::SetEnergy(int i, int e, double r, double bg)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  printf("ProjSpectrum::SetEnergy\n");
#endif

  energy[i] = e;
  ratio[i] = r;
  background[i] = bg;
  return 0;
}

/*
int PrjSpectrum_class::SetSpectrum(int* e, double* r, double* bg)
{
  for(unsigned int i = 0; i < NumberOfEnergies; i++) {
    energy[i] = e[i];
    ratio[i] = r[i];
    background[i] = bg[i];
  }

  return 0;
}

// Energy Spectrum

int PrjSpectrum_class::GetNumberOfEnergies(unsigned int* NOE)
{
  *NOE = NumberOfEnergies;

	return 0;
}


int PrjSpectrum_class::GetEnergy(int index, int* e, double* r, double* bg)
{
  *e = energy[index];
	*r = ratio[index];
	*bg = background[index];
	
	return 0;
}
*/

// Noise

ProjNoise::ProjNoise()
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  printf("ProjNoise::ProjNoise\n");
#endif

	IsQuan = IsScat = IsAdd = IsMult = false;
};

// Quantum

void ProjNoise::SetQuantum(double pMean, double pCalibration, int pGantry)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  printf("ProjNoise::SetQuantum\n");
#endif

	IsQuan = true;

	QuantumMean = pMean;
	QuantumCalibration = pCalibration;
	QuantumGantry = pGantry;
}


// Scatter

void ProjNoise::SetScatter(double pPeak, double pWidth)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  printf("ProjNoise::SetScatter\n");
#endif

	IsScat = true;

	ScatterPeak = pPeak;
	ScatterWidth = pWidth;
}


// Additive

void ProjNoise::SetAdditive(double pMean, double pStdDev)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  printf("ProjNoise::SetAdditive\n");
#endif

	IsAdd = true;

	AdditiveMean = pMean;
	AdditiveStdDev = pStdDev;
}


// Multiplicative

void ProjNoise::SetMultiplicative(double pMean, double pStdDev)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  printf("ProjNoise::SetMultiplicative\n");
#endif

	IsMult = true;

	MultiplicativeMean = pMean;
	MultiplicativeStdDev = pStdDev;
}


// Noise

// Quantum
/*
int PrjNoise_class::GetQuantum(double& m, double& cm, double& g)
{
  if(!IsQuan) {
    return -1;
  }

	m = Quantum_Mean;
	cm = Quantum_Calibration;
	g = Quantum_Gantry;

  return 0;
}


// Scatter

int PrjNoise_class::GetScatter(double& p, double& w)
{
  if(!IsScat) {
    return -1;
  }

	p = Scatter_Peak;
	w = Scatter_Width;

  return 0;
}


// Additive

int PrjNoise_class::GetAdditive(double& m, double& sd)
{
  if(!IsAdd) {
    return -1;
  }

	m = Additive_Mean;
	sd = Additive_Std_Dev;

  return 0;
}


// Multiplicative

int PrjNoise_class::GetMultiplicative(double& m, double& sd)
{
  if(!IsMult) {
    return -1;
  }

	m = Multiplicative_Mean;
	sd = Multiplicative_Std_Dev;

  return 0;
}
*/

const char* DIGFileSnarkProj::TypeStr = "SNARK05 prjfil";
const char* DIGFileSnarkProj::SchemaStr = "DIGFileSnarkProj.xsd";

bool DIGFileSnarkProj::XMLStringsInitialized = false;

const XMLCh* DIGFileSnarkProj::AppHeaderXMLStr;
const XMLCh* DIGFileSnarkProj::GeomHeaderXMLStr;
const XMLCh* DIGFileSnarkProj::DivergentXMLStr;
const XMLCh* DIGFileSnarkProj::RadiusXMLStr;
const XMLCh* DIGFileSnarkProj::SourceDetXMLStr;
const XMLCh* DIGFileSnarkProj::DetTypeXMLStr;
const XMLCh* DIGFileSnarkProj::ParallelXMLStr;
const XMLCh* DIGFileSnarkProj::ProjTypeXMLStr;
const XMLCh* DIGFileSnarkProj::NoiseHeaderXMLStr;
const XMLCh* DIGFileSnarkProj::QuantumXMLStr;
const XMLCh* DIGFileSnarkProj::MeanXMLStr;
const XMLCh* DIGFileSnarkProj::CalibMeasXMLStr;
const XMLCh* DIGFileSnarkProj::GantryXMLStr;
const XMLCh* DIGFileSnarkProj::ScatterXMLStr;
const XMLCh* DIGFileSnarkProj::PeakXMLStr;
const XMLCh* DIGFileSnarkProj::WidthXMLStr;
const XMLCh* DIGFileSnarkProj::AdditiveXMLStr;
const XMLCh* DIGFileSnarkProj::StdDevXMLStr;
const XMLCh* DIGFileSnarkProj::MultipXMLStr;
const XMLCh* DIGFileSnarkProj::SpectrumHeaderXMLStr;
const XMLCh* DIGFileSnarkProj::NumOfEnerXMLStr;
const XMLCh* DIGFileSnarkProj::EnergyLevelXMLStr;
const XMLCh* DIGFileSnarkProj::EnergyXMLStr;
const XMLCh* DIGFileSnarkProj::RatioXMLStr;
const XMLCh* DIGFileSnarkProj::BackgroundXMLStr;

DIGFileSnarkProj::DIGFileSnarkProj()
{

  // XML string initialization
  if(!XMLStringsInitialized) {

	  AppHeaderXMLStr      = XMLString::transcode("application_header");
	  GeomHeaderXMLStr     = XMLString::transcode("geometric_header");
	  DivergentXMLStr      = XMLString::transcode("divergent");
	  RadiusXMLStr         = XMLString::transcode("radius");
	  SourceDetXMLStr      = XMLString::transcode("source_detector_distance");
	  DetTypeXMLStr        = XMLString::transcode("detector_type");
	  ParallelXMLStr       = XMLString::transcode("parallel");
	  ProjTypeXMLStr       = XMLString::transcode("projection_type");
	  NoiseHeaderXMLStr    = XMLString::transcode("noise_header");
	  QuantumXMLStr        = XMLString::transcode("quantum");
	  MeanXMLStr           = XMLString::transcode("mean");
	  CalibMeasXMLStr      = XMLString::transcode("calibration_measurement");
	  GantryXMLStr         = XMLString::transcode("gantry_arrangement");
	  ScatterXMLStr        = XMLString::transcode("scatter");
	  PeakXMLStr           = XMLString::transcode("peak");
	  WidthXMLStr          = XMLString::transcode("width");
	  AdditiveXMLStr       = XMLString::transcode("additive");
	  StdDevXMLStr         = XMLString::transcode("standard_deviation");
	  MultipXMLStr         = XMLString::transcode("multiplicative");
	  SpectrumHeaderXMLStr = XMLString::transcode("spectrum_header");
	  NumOfEnerXMLStr      = XMLString::transcode("number_of_energies");
	  EnergyLevelXMLStr    = XMLString::transcode("energy_level");
	  EnergyXMLStr         = XMLString::transcode("energy");
	  RatioXMLStr          = XMLString::transcode("ratio");
	  BackgroundXMLStr     = XMLString::transcode("background");
  }

  Spectrum = NULL;
  Geometry = NULL;
}	


DIGFileSnarkProj::~DIGFileSnarkProj()
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:~DIGFileSnarkProj\n");
#endif

  if(Geometry != NULL) {
	  delete Geometry;
  }

  if(Spectrum != NULL) {
	  delete Spectrum;
  }
}

////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////


///////////////////////////////////////////////
// main header reading
///////////////////////////////////////////////

int DIGFileSnarkProj::GetMainHeader(ProjFileMH* MainHeader)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetMainHeader\n");
#endif

  // read title
  const char* Title;

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
  printf("Getting Main Header Title\n");
#endif

  if(GetTitle(&Title) != 0) {
    printf("Error reading Main Header Title\n");
    return -1;
  }

  //printf("Main Header Title: %s\n", Title);
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
  printf("Seting Main Header Title\n");
#endif

  if(MainHeader->Title.Set(Title) != 0) {
    printf("Error seting Main Header Title\n");
    return -1;
  }

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
  printf("Seting Main Header Dimensions\n");
#endif

  if(GetDimensions(&(MainHeader->Dimensions)) != 0) {
    printf("Error reading Main Header Dimensions\n");
    return -1;
  }

  //printf("Main Header Dimensions: %d\n", MainHeader->Dimensions);

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
  printf("Seting Main Header Sampling\n");
#endif

  if(GetSampling(&(MainHeader->Sampling)) != 0) {
    printf("Error reading Main Header Sampling\n");
    return -1;
  }

  //printf("Main Header Sampling: %f\n", MainHeader->Sampling);

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
  printf("Getting Main Header comments\n");
#endif

  const char* Comment;

  if(GetComment(&Comment) != 0) {
    printf("Error reading Main Header Comment\n");
    return -1;
  }

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
  printf("Seting Main Header Comments\n");
#endif

  //printf("Main Header Comment: %s\n", Comment);

  if(MainHeader->Comment.Set(Comment) != 0) {
    printf("Error Seting Main Header Comment\n");
    return -1;
  }

  return 0;
}

int DIGFileSnarkProj::GetAppHeader(ProjFileAH* pAppHeader)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetAppHeader\n");
#endif

  // application header comment
  const char* Comment;

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
  printf("Getting Application Header Comments\n");
#endif

  if(GetAppComment(&Comment) != 0) {
    printf("Error reading Application Header Comment\n");
    return -1;
  }

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
  printf("Setting Application Header Comments\n");
#endif

  //printf("Application Header Comment: %s\n", Comment);

  if(pAppHeader->Comment.Set(Comment) != 0) {
    printf("Error Seting Application Header Comment\n");
    return -1;
  }

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
  printf("Getting Application Header Geometry Type\n");
#endif

  // geometry
  if(GetGeometryType(&(pAppHeader->GeometryType)) != 0) {
    printf("Error reading Application Header Geometry Type\n");
    return -1;
  }

  //printf("Application Header Geometry Type: %s\n", (pAppHeader->GeometryType == GT_DIVERGENT) ? "Divergent": "Parallel");

  if(pAppHeader->GeometryType == GT_DIVERGENT) {  // divergent

    if(GetGeometry(&(pAppHeader->Radius), &(pAppHeader->SrcDetDistance), &(pAppHeader->DetectorType)) != 0) {
      printf("Error reading Application Header Geometry\n");
      return -1;
    }

    //printf("Application Header Geometry Radius: %f\n", pAppHeader->Radius);
    //printf("Application Header Geometry SrcDetDistance: %f\n", pAppHeader->SrcDetDistance);

  }
  else { // parallel

    if(GetGeometry(&(pAppHeader->DetectorType), &(pAppHeader->ProjType)) != 0) {
      printf("Error reading Application Header Geometry\n");
      return -1;
    }

    //printf("Application Header Geometry Projection Type: %s\n", (pAppHeader->ProjType == PT_STRIP) ? "Strip": "Line");

  }

  /*
  switch(pAppHeader->DetectorType) {

  case DT_TANGENT:
    printf("Application Header Geometry Detector Type: TANGENT\n");
    break;

  case DT_ARC:
    printf("Application Header Geometry Detector Type: ARC\n");
    break;

  case DT_UNIFORM:
    printf("Application Header Geometry Detector Type: UNIFORM\n");
    break;

  case DT_VARIABLE:
    printf("Application Header Geometry Detector Type: VARIABLE\n");
    break;
  }
  */

  // spectrum

  if(GetNoOfEnergies(&(pAppHeader->NoOfEnergies)) != 0) {
    printf("Error reading Application Header Spectrum Number Of Energies\n");
    return -1;
  }

  //printf("Application Header Spectrum Number Of Energies: %u\n", pAppHeader->NoOfEnergies);

  pAppHeader->Energy = new int[pAppHeader->NoOfEnergies];
  pAppHeader->Ratio = new double[pAppHeader->NoOfEnergies];
  pAppHeader->Background = new double[pAppHeader->NoOfEnergies];

  for(unsigned int k = 0; k < pAppHeader->NoOfEnergies; k++) {
    if(GetSpectrum(k, &(pAppHeader->Energy[k]), &(pAppHeader->Ratio[k]), &(pAppHeader->Background[k])) != 0) {
      printf("Error reading Application Header Spectrum Energy\n");
      return -1;
    }

    //printf("Application Header Spectrum Energy: %u %i %f %f\n", k, pAppHeader->Energy[k], pAppHeader->Ratio[k], pAppHeader->Background[k]);

  }

  // noise
  if(HasNoiseQuantum(&(pAppHeader->QuantumFlag)) != 0) {
    printf("Error reading Application Header Noise Quantum Flag\n");
    return -1;
  }

  if(pAppHeader->QuantumFlag) {

    if(GetNoiseQuantum(&(pAppHeader->QuantumM), &(pAppHeader->QuantumCM), &(pAppHeader->QuantumGA)) != 0) {
      printf("Error reading Application Header Noise Quantum Flag\n");
      return -1;
    }

    //printf("Application Header Noise Quantum: %f %f %i\n", pAppHeader->QuantumM, pAppHeader->QuantumCM, pAppHeader->QuantumGA);
  }

  if(HasNoiseScatter(&(pAppHeader->ScatterFlag)) != 0) {
    printf("Error reading Application Header Noise Scatter Flag\n");
    return -1;
  }

  if(pAppHeader->ScatterFlag) {

    if(GetNoiseScatter(&(pAppHeader->ScatterP), &(pAppHeader->ScatterW)) != 0) {
      printf("Error reading Application Header Noise Scatter\n");
      return -1;
    }

    //printf("Application Header Noise Scatter: %f %f\n", pAppHeader->ScatterP, pAppHeader->ScatterW);
  }

  if(HasNoiseAdditive(&(pAppHeader->AdditiveFlag)) != 0) {
    printf("Error reading Application Header Noise Additive Flag\n");
    return -1;
  }

  if(pAppHeader->AdditiveFlag) {
    if(GetNoiseAdditive(&(pAppHeader->AdditiveM), &(pAppHeader->AdditiveSD)) != 0) {
      printf("Error reading Application Header Noise Additive\n");
      return -1;
    }

    //printf("Application Header Noise Additive: %f %f\n", pAppHeader->AdditiveM, pAppHeader->AdditiveSD);
  }

  if(HasNoiseMultiplicative(&(pAppHeader->MultiplicativeFlag)) != 0) {
    printf("Error reading Application Header Noise Multiplicative Flag\n");
    return -1;
  }

  if(pAppHeader->MultiplicativeFlag) {

    if(GetNoiseMultiplicative(&(pAppHeader->MultiplicativeM), &(pAppHeader->MultiplicativeSD)) != 0) {
      printf("Error reading Application Header Noise Multiplicative\n");
      return -1;
    }

    //printf("Application Header Noise Multiplicative: %f %f\n", pAppHeader->MultiplicativeM, pAppHeader->MultiplicativeSD);
  }

  return 0;
}

///////////////////////////////////////////////
// Projection header reading
///////////////////////////////////////////////

int DIGFileSnarkProj::GetProjHeader(ProjFilePrjH* pProjHeader)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetProjHeader\n");
#endif

  // read projection number

  if(GetProjNo(&(pProjHeader->ProjNo)) != 0) {
    printf("Error reading Projection Number\n");
    return -1;
  }

  //printf("Projection Number: %u\n", pProjHeader->ProjNo);

  // read angle

  if(GetProjAngle(&(pProjHeader->Angle)) != 0) {
    printf("Error reading Projection Angle\n");
    return -1;
  }

  //printf("Array Set Parameters: %f\n", pProjHeader->Angle);

  // read comment
  const char* Comment;

  if(GetProjComment(&Comment) != 0) {
    printf("Error reading Projection Comment\n");
    return -1;
  }

  //printf("Projection Comment: %s\n", Comment);

  if(pProjHeader->Comment.Set(Comment) != 0) {
    printf("Error Seting Projection Comment\n");
    return -1;
  }

  return 0;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Writing
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Open for writing
///////////////////////////////////////////////////////////////////////////////

int DIGFileSnarkProj::Open(const char* pFileName, ProjFileMH* pMainHeader)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:Open\n");
#endif

  if(
    Open(
      pFileName,                  // file name
      pMainHeader->Title.Get(),   // title
      pMainHeader->Dimensions,    // dimensions
      pMainHeader->Sampling,      // sampling
      pMainHeader->Comment.Get()  // comment 
    ) != 0
  )
  {
    printf("Error opening file\n");
    return -1;
  }
  return 0;
}

// open for writing
int DIGFileSnarkProj::Open(
  const char*   pFileName,     // File Name
  const char*   pTitle,        // Title
  unsigned int  pNumberOfRays, // USRAYS
  double        pSampling,     // PINC
  const char*   pComments      // Comments
) 
{
  //const char*           Schema;
  DIGDimensions  dim;
  DIGSampling    sampX;
  DIGSampling    samp = { 0.0, 0.0, 0.0 };

  dim.x = pNumberOfRays;
  dim.y = 1;
  dim.z = 1;

  sampX.x = pSampling;
  sampX.y = 0.0;
  sampX.z = 0.0;

  Noise.IsQuan = false;
  Noise.IsAdd = false;
  Noise.IsScat = false;
  Noise.IsMult = false;

  return DIGFile::Open(
    pFileName,            // File Name
    SchemaStr,            // Schema
    pTitle,               // Title
    TypeStr,              // Type,
    1,                    // Chanels
    DIGValueType_REAL,    // ValueType
    DIGDataType_DOUBLE,   // DataType
    DIGDataFormat_BINARY, // DataFormat
    DIGGrid_SC,           // Grid
    DIGBasis_VORONOI,     // Basis
    DIGUnit_UNSPECIFIED,  // Unit
    &dim,                 // Dimensions
    &sampX,               // SamplingX
    &samp,                // SamplingY
    &samp,                // SamplingZ
    pComments,            // Comments
    ""                    // Other Unit
  );
};


///////////////////////////////////////////////
// application header writing
///////////////////////////////////////////////

int DIGFileSnarkProj::SetAppHeader(ProjFileAH* pAppHeader)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:SetAppHeader\n");
#endif

  const char* Comment;

  if(pAppHeader->Comment.Get(&Comment)) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
  printf("** Error: Getting Application Header Comments\n");
#endif
    return -1;
  }

  SetAppComment(Comment);

  switch(pAppHeader->GeometryType) {
  case GT_DIVERGENT:
    //pOut->SetGeometry(double r, double d, DetectorTypeEnum t);
    if(SetGeometry(pAppHeader->Radius, pAppHeader->SrcDetDistance, pAppHeader->DetectorType) != 0) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
      printf("** Error: Setting Application Header Geometry\n");
#endif
      return -1;
    }
    break;

  case GT_PARALLEL:
    if(SetGeometry(pAppHeader->DetectorType, pAppHeader->ProjType) != 0) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
      printf("** Error: Setting Application Header Geometry\n");
#endif
      return -1;
    }
    break;

  default:

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
      printf("** Error: Setting Application Header - Unknown Geometry\n");
#endif
    ;// Error
  }

  if(SetSpectrum(pAppHeader->NoOfEnergies, pAppHeader->Energy, pAppHeader->Ratio, pAppHeader->Background) != 0) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: Setting Application Header Spectrum\n");
#endif
    return -1;
  }

  if(pAppHeader->QuantumFlag) {
	  if(SetNoiseQuantum(pAppHeader->QuantumM, pAppHeader->QuantumCM, pAppHeader->QuantumGA) != 0) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: Setting Application Header Noise - Quantum\n");
#endif
      return -1;
    }
  }

  if(pAppHeader->ScatterFlag) {
	  if(SetNoiseScatter(pAppHeader->ScatterP, pAppHeader->ScatterW) != 0) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: Setting Application Header Noise - Scatter\n");
#endif
      return -1;
    }
  }

  if(pAppHeader->AdditiveFlag) {
	  if(SetNoiseAdditive(pAppHeader->AdditiveM, pAppHeader->AdditiveSD) != 0) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: Setting Application Header Noise - Additive\n");
#endif
      return -1;
    }
  }

  if(pAppHeader->MultiplicativeFlag) {
	  if(SetNoiseMultiplicative(pAppHeader->MultiplicativeM, pAppHeader->MultiplicativeSD) != 0) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: Setting Application Header Noise - Multiplicative\n");
#endif
      return -1;
    }
  }

  return 0;
}

///////////////////////////////////////////////
// projection appending
///////////////////////////////////////////////

int DIGFileSnarkProj::AppendProj(ProjFilePrjH* pProjHeader, const double* pData)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:AppendProj\n");
#endif

  return AppendProj(
    pProjHeader->ProjNo,
    pProjHeader->Angle,
    pProjHeader->Comment.Get(),
    pData
  );
}

int DIGFileSnarkProj::AppendProj(
  unsigned int  pProjNo,
  double        pAngle,
  const char*   pComment,
  const double* pData
) 
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:AppendProj\n");
#endif

  int ret;
  char ProjNoStr[8];   // Projection Number as a String
  char AngleStr[256];  // angle as a String

	AngleStr[0] = 0;
	sprintf(AngleStr, "%.20f", pAngle);

	ProjNoStr[0] = 0;
	sprintf(ProjNoStr, "%d", pProjNo);

  if((ret = AppendArraySet("projection", ProjNoStr, AngleStr, pComment)) != 0) {
    return ret;
  }

  return AppendArray(0, "", pData);
}

////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////

// open for reading

int DIGFileSnarkProj::Open(const char* pFileName)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:Open\n");
#endif

  Noise.IsQuan = false;
  Noise.IsAdd = false;
  Noise.IsScat = false;
  Noise.IsMult = false;

  if(DIGFile::Open(pFileName) != 0) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: Openning DIG file\n");
#endif
    return -1; // error opening dig file
  }

  if(strcmp(MainHeader.Type.Get(), TypeStr) != 0) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: Not a Snark05 prjfil file\n");
#endif
    return -2; // not a DIGFileSnarkProj
  }

  return 0;
}

// comment

int DIGFileSnarkProj::GetAppComment(const char** pComment)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetAppComment\n");
#endif

  return AppComment.Get(pComment);
} 

int DIGFileSnarkProj::GetGeometryType(GeometryTypeEnum* pGeometryType)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetGeometryType\n");
#endif

  *pGeometryType = Geometry->GeometryType;
  return 0;
}

// divergent
int DIGFileSnarkProj::GetGeometry(double* pRadius, double* pSrcDetDistance, DetectorTypeEnum* pDetectorType)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetGeometry\n");
#endif

  if(Geometry == NULL) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: Undefines Geometry\n");
#endif
    return -1; // error
  }

	*pRadius = Geometry->Radius;
	*pSrcDetDistance = Geometry->SrcDetDistance;
	*pDetectorType = Geometry->DetectorType;

  return 0;
}

// parallel
int DIGFileSnarkProj::GetGeometry(DetectorTypeEnum* pDetectorType, ProjTypeEnum* pProjType)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetGeometry\n");
#endif

  if(Geometry == NULL) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: Undefines Geometry\n");
#endif
    return -1; // error
  }

	*pDetectorType = Geometry->DetectorType;
	*pProjType = Geometry->ProjType;

  return 0;
}


int DIGFileSnarkProj::GetNoOfEnergies(unsigned int* NumbOfEnergies)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetNoOfEnergies\n");
#endif

  *NumbOfEnergies = Spectrum->NumberOfEnergies;

  return 0;
}

int DIGFileSnarkProj::GetSpectrum(unsigned int i, int* e, double* r, double* bg)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetSpectrum\n");
#endif

  if(Spectrum == NULL) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: GetSpectrum - Undefines Spectrum\n");
#endif
    return -1; // error
  }

  if(i >= Spectrum->NumberOfEnergies) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: GetSpectrum - invalid parameter\n");
#endif
    return -2; // error
  }

  *e = Spectrum->energy[i];
  *r = Spectrum->ratio[i];
  *bg = Spectrum->background[i];

  return 0;
}


int DIGFileSnarkProj::HasNoiseQuantum(bool* a)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:HasNoiseQuantum\n");
#endif

  *a = Noise.IsQuan; 
  return 0;
}


int DIGFileSnarkProj::GetNoiseQuantum(double* pMean, double* pCalibration, int* pGantry)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetNoiseQuantum\n");
#endif

  if(Noise.IsQuan != true) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: GetNoiseQuantum - Quantum Noise Undefined\n");
#endif
    return -1; // error
  }

	*pMean = Noise.QuantumMean;
  *pCalibration = Noise.QuantumCalibration;
  *pGantry = Noise.QuantumGantry;

  return 0;
}


// Scatter

int DIGFileSnarkProj::HasNoiseScatter(bool* a)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:HasNoiseScatter\n");
#endif

  *a = Noise.IsScat; 
  return 0;
}

int DIGFileSnarkProj::GetNoiseScatter(double* pPeak, double* pWidth)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetNoiseScatter\n");
#endif

  if(Noise.IsScat != true) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: GetNoiseScatter - Scatter Noise Undefined\n");
#endif
    return -1; // error
  }

	*pPeak = Noise.ScatterPeak;
  *pWidth = Noise.ScatterWidth;

  return 0;
}


// Additive

int DIGFileSnarkProj::HasNoiseAdditive(bool* a)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:HasNoiseAdditive\n");
#endif

  *a = Noise.IsAdd; 
  return 0;
}


int DIGFileSnarkProj::GetNoiseAdditive(double* pMean, double* pStdDev)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetNoiseAdditive\n");
#endif

  if(Noise.IsAdd != true) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: GetNoiseAdditive - Additive Noise Undefined\n");
#endif
    return -1; // error
  }

	*pMean = Noise.AdditiveMean;
  *pStdDev = Noise.AdditiveStdDev;

  return 0;
}


// Multiplicative

int DIGFileSnarkProj::HasNoiseMultiplicative(bool* a)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:HasNoiseMultiplicative\n");
#endif

  *a = Noise.IsMult; 
  return 0;
}


int DIGFileSnarkProj::GetNoiseMultiplicative(double* pMean, double* pStdDev)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:GetNoiseMultiplicative\n");
#endif

  if(Noise.IsMult != true) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: GetNoiseMultiplicative - Multiplicative Noise Undefined\n");
#endif
    return -1; // error
  }

	*pMean = Noise.MultiplicativeMean;
  *pStdDev = Noise.MultiplicativeStdDev;

  return 0;
}



// comment

int DIGFileSnarkProj::SetAppComment(const char* pComment)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:SetAppComment\n");
#endif

  return AppComment.Set(pComment);
} 

// divergent
int DIGFileSnarkProj::SetGeometry(double pRadius, double pSrcDetDistance, DetectorTypeEnum pDetectorType)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:SetGeometry\n");
#endif

  if(Geometry != NULL) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: SetGeometry - Geometry Aleready Defined\n");
#endif
    return -1; // already set
  }

	Geometry = new ProjGeometry(pRadius, pSrcDetDistance, pDetectorType);

  return 0;
}

// parallel
int DIGFileSnarkProj::SetGeometry(DetectorTypeEnum pDetectorType, ProjTypeEnum pProjType)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:SetGeometry\n");
#endif

  if(Geometry != NULL) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: SetGeometry - Geometry Aleready Defined\n");
#endif
    return -1; // already set
  }

	Geometry = new ProjGeometry(pDetectorType, pProjType);

  return 0;
}

int DIGFileSnarkProj::SetSpectrum(unsigned int eNum, int* e, double* r, double* bg)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:SetSpectrum\n");
#endif

  if(Spectrum != NULL) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: SetSpectrum - Spectrum Aleready Defined\n");
#endif
    return -1; // already set
  }

	Spectrum = new ProjSpectrum(eNum);

  for(unsigned int i = 0; i < eNum; i++) {
    Spectrum->energy[i] = e[i];
    Spectrum->ratio[i] = r[i];
    Spectrum->background[i] = bg[i];
  }

  return 0;
}


int DIGFileSnarkProj::SetNoiseQuantum(double pMean, double pCalibration, int pGantry)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:SetNoiseQuantum\n");
#endif

  if(Noise.IsQuan) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: SetNoiseQuantum - Noise Quantum Aleready Defined\n");
#endif
    return -1; // already set
  }

	Noise.SetQuantum(pMean, pCalibration, pGantry);

  return 0;
}


// Scatter

int DIGFileSnarkProj::SetNoiseScatter(double pPeak, double pWidth)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:SetNoiseScatter\n");
#endif

  if(Noise.IsScat) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: SetNoiseScatter - Noise Scatter Aleready Defined\n");
#endif
    return -1; // already set
  }

	Noise.SetScatter(pPeak, pWidth);

  return 0;
}


// Additive

int DIGFileSnarkProj::SetNoiseAdditive(double pMean, double pStdDev)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:SetNoiseAdditive\n");
#endif

  if(Noise.IsAdd) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: SetNoiseAdditive - Noise Additive Aleready Defined\n");
#endif
    return -1; // already set
  }

	Noise.SetAdditive(pMean, pStdDev);

  return 0;
}


// Multiplicative

int DIGFileSnarkProj::SetNoiseMultiplicative(double pMean, double pStdDev)
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:SetNoiseMultiplicative\n");
#endif

  if(Noise.IsMult) {

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 0
    printf("** Error: SetNoiseMultiplicative - Noise Multiplicative Aleready Defined\n");
#endif
    return -1; // already set
  }

	Noise.SetMultiplicative(pMean, pStdDev);

  return 0;
}


/////////////////////////////////////////////////////////////////////
//
// Internal Methods
//
/////////////////////////////////////////////////////////////////////

// NOTE: error handling is not included
int DIGFileSnarkProj::ParseApplicationHeader()
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:ParseApplicationHeader\n");
#endif

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
    printf("Parsing Application Header\n");
#endif

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
    printf("Getting Comments\n");
#endif

  // get Comment Texts
  char* tCom = DIGFileSnarkPrjEmptyStr;

  DOMNodeList* CommentsList = ApplicationHeaderElement->getElementsByTagName(CommentsXMLStr);

  int tNumC = CommentsList->getLength();
  if(tNumC > 0)	{

    DOMElement* CommentsElement = (DOMElement*) CommentsList->item(0);

    if(GetElementText(CommentsElement, &tCom) != 0) {
      ;// allow empty comments
    }
  }

  AppComment.Set(tCom);


#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
    printf("Getting Geometric Header\n");
#endif

  // get geometric header

  DOMNodeList* GeomHeaderList = ApplicationHeaderElement->getElementsByTagName(GeomHeaderXMLStr);
  DOMElement* GeomHeaderElement = (DOMElement*) GeomHeaderList->item(0);
	
  // get divergent
	
  DOMNodeList* DivergentList = GeomHeaderElement->getElementsByTagName(DivergentXMLStr);
	if(DivergentList->getLength() > 0) {
		DOMElement* DivergentElement = (DOMElement*) DivergentList->item(0);

		// get radius, source detector distance, detector type

		double tRadius = 0;
		GetDoubleAttributeValue(DivergentElement, RadiusXMLStr, &tRadius);
				
		double tSrcDetDist = 0;
		GetDoubleAttributeValue(DivergentElement, SourceDetXMLStr, &tSrcDetDist);
		
		DOMNodeList* DetectList = DivergentElement->getElementsByTagName(DetTypeXMLStr);
		DOMElement* DetectElement = (DOMElement*) DetectList->item(0);
		
		char* tstring;
		DetectorTypeEnum tDet;

		GetElementText(DetectElement, &tstring);

    if(strcmp(tstring, "tangent") == 0) {
      tDet = DT_TANGENT;
    }

    if(strcmp(tstring, "arc") == 0) {
      tDet = DT_ARC;
    }
		
		Geometry = new ProjGeometry(tRadius, tSrcDetDist, tDet);
	}
	else {
		// get parallel tag

		DOMNodeList* ParallelList = GeomHeaderElement->getElementsByTagName(ParallelXMLStr);
		DOMElement* ParallelElement = (DOMElement*) ParallelList->item(0);
		
		// get detector type

		DOMNodeList* DetectList = ParallelElement->getElementsByTagName(DetTypeXMLStr);
		DOMElement* DetectElement = (DOMElement*) DetectList->item(0);
		
		char *tstring;
		DetectorTypeEnum tDet;

		GetElementText(DetectElement, &tstring);

    if(strcmp(tstring, "uniform") == 0) {
      tDet = DT_UNIFORM;
    }

    if(strcmp(tstring, "variable") == 0) {
      tDet = DT_VARIABLE;
    }
		
		// get projection type

		DOMNodeList* ProjectList = ParallelElement->getElementsByTagName(ProjTypeXMLStr);
		DOMElement* ProjectElement = (DOMElement*) ProjectList->item(0);
		
		ProjTypeEnum tProj;
		
		GetElementText(ProjectElement, &tstring);

    if(strcmp(tstring, "line") == 0) {
      tProj = PT_LINE;
    }
    else if(strcmp(tstring, "strip") == 0) {
      tProj = PT_STRIP;
    }

		
		Geometry = new ProjGeometry(tDet, tProj);
	}

#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
    printf("Getting Noise Header\n");
#endif

	// get noise header

	DOMNodeList* NoiseHeaderList = ApplicationHeaderElement->getElementsByTagName(NoiseHeaderXMLStr);
	DOMElement* NoiseHeaderElement = (DOMElement*) NoiseHeaderList->item(0);
	
	// get quantum
	
	DOMNodeList* QuantumList = NoiseHeaderElement->getElementsByTagName(QuantumXMLStr);
	if(QuantumList->getLength() > 0) {
		DOMElement* QuantumElement = (DOMElement*) QuantumList->item(0);
		
		// get mean, calibration measurement, gantry arrangement
		
		double tMean = 0, tCal = 0;
		int tGan = 0;

		GetDoubleAttributeValue(QuantumElement, MeanXMLStr, &tMean);
		GetDoubleAttributeValue(QuantumElement, CalibMeasXMLStr, &tCal);
		GetIntAttributeValue(QuantumElement, GantryXMLStr, &tGan);
		
		Noise.SetQuantum(tMean, tCal, tGan);
	}
	
	// get scatter
	
	DOMNodeList* ScatterList = NoiseHeaderElement->getElementsByTagName(ScatterXMLStr);
	if(ScatterList->getLength() > 0) {
		DOMElement* ScatterElement = (DOMElement*) ScatterList->item(0);
		
		// get peak, width
		
		double tPeak = 0, tWidth = 0;

		GetDoubleAttributeValue(ScatterElement, PeakXMLStr, &tPeak);
		GetDoubleAttributeValue(ScatterElement, WidthXMLStr, &tWidth);
		
		Noise.SetScatter(tPeak, tWidth);
	}

	// get additive
	
	DOMNodeList* AdditList = NoiseHeaderElement->getElementsByTagName(AdditiveXMLStr);
	if(AdditList->getLength() > 0) {
		DOMElement* AdditElement = (DOMElement*) AdditList->item(0);
		
		// get mean, calibration measurement, gantry arrangement
		
		double tMean = 0, tStd = 0;

		GetDoubleAttributeValue(AdditElement, MeanXMLStr, &tMean);
		GetDoubleAttributeValue(AdditElement, StdDevXMLStr, &tStd);
		
		Noise.SetAdditive(tMean, tStd);
	}

	// get multiplicative
	
	DOMNodeList* MultipList = NoiseHeaderElement->getElementsByTagName(MultipXMLStr);
	if(MultipList->getLength() > 0) {
		DOMElement* MultipElement = (DOMElement*) MultipList->item(0);
		
		// get mean, calibration measurement, gantry arrangement
		
		double tMean = 0, tStd = 0;

		GetDoubleAttributeValue(MultipElement, MeanXMLStr, &tMean);
		GetDoubleAttributeValue(MultipElement, StdDevXMLStr, &tStd);
		
		Noise.SetMultiplicative(tMean, tStd);
	}
	
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 1
    printf("Getting Specrum Header\n");
#endif

	// get spectrum header

	DOMNodeList* SpectHeaderList = ApplicationHeaderElement->getElementsByTagName(SpectrumHeaderXMLStr);
	DOMElement* SpectHeaderElement = (DOMElement*) SpectHeaderList->item(0);

	// get number of energies
	
	unsigned int tNumE = 0;
	
	GetUintAttributeValue(SpectHeaderElement, NumOfEnerXMLStr, &tNumE);
	//SetEnergyNumber(tNumE);

  Spectrum = new ProjSpectrum(tNumE);
	
	// get energy level triplets: energy, ratio, background
	
	DOMNodeList* EnergyList = SpectHeaderElement->getElementsByTagName(EnergyLevelXMLStr);
	
	// NOTE: Error if EnergyList->getLength() != tNumE
	
	int tEner = 0;
	double tRatio = 0, tBack = 0;
	DOMElement* EnergyElement;
	
	for(unsigned int i = 0; i < EnergyList->getLength(); i++) {
		EnergyElement = (DOMElement*) EnergyList->item(i);
		
		GetIntAttributeValue(EnergyElement, EnergyXMLStr, &tEner);
		GetDoubleAttributeValue(EnergyElement, RatioXMLStr, &tRatio);
		GetDoubleAttributeValue(EnergyElement, BackgroundXMLStr, &tBack);

		Spectrum->SetEnergy(i, tEner, tRatio, tBack);		
	}
			
	return 0;
}


int DIGFileSnarkProj::CreateApplicationHeader()
{
#if DIGFILESNARKPROJ_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkProj:CreateApplicationHeader\n");
#endif

  char valStr[20];

  XMLCh* tmpXMLStr;

  // create main header node
  ApplicationHeaderElement = Document->createElement(AppHeaderXMLStr);

  // attributes

    // there is no attributes  

  // children


  // define comments
  const char* Comment;

  AppComment.Get(&Comment);

	if(Comment != NULL) {
    
    // create comments node
    DOMElement* Comm = Document->createElement(CommentsXMLStr);

    // create comments text node
    tmpXMLStr = XMLString::transcode(Comment);
    DOMText* CommentText = Document->createTextNode(tmpXMLStr);
    //delete tmpXMLStr;
    Comm->appendChild(CommentText);

    // append comments to appication header
    ApplicationHeaderElement->appendChild(Comm);
  }


  // geometry

  // create geometry node
  DOMElement* Geom = Document->createElement(GeomHeaderXMLStr);

	if(Geometry->GeometryType == GT_DIVERGENT) {

    DOMElement* Diver = Document->createElement(DivergentXMLStr);

    // define radius
    sprintf(valStr, "%f", Geometry->Radius);
    tmpXMLStr = XMLString::transcode(valStr);
    Diver->setAttribute(RadiusXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // define source detector distanse
    sprintf(valStr, "%f", Geometry->SrcDetDistance);
    tmpXMLStr = XMLString::transcode(valStr);
    Diver->setAttribute(SourceDetXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // define detector type
    DOMElement* DetT = Document->createElement(DetTypeXMLStr);

    // create detector type text node
    if(Geometry->DetectorType == DT_TANGENT) {
      tmpXMLStr = XMLString::transcode("tangent");
    }
    else {
      tmpXMLStr = XMLString::transcode("arc");
    }

    DOMText* DetTText = Document->createTextNode(tmpXMLStr);
    //delete tmpXMLStr;
    DetT->appendChild(DetTText);

    // append detector type to divergent
    Diver->appendChild(DetT);

    // append divergent to geometry
    Geom->appendChild(Diver);
  }
  else {

    DOMElement* Para = Document->createElement(ParallelXMLStr);

    // define detector type
    DOMElement* DetT = Document->createElement(DetTypeXMLStr);

    if(Geometry->DetectorType == DT_UNIFORM) {
      tmpXMLStr = XMLString::transcode("uniform");
    }
    else {
      tmpXMLStr = XMLString::transcode("variable");
    }

    DOMText* DetTText = Document->createTextNode(tmpXMLStr);
    //delete tmpXMLStr;
    DetT->appendChild(DetTText);

    // append detector type to parallel
    Para->appendChild(DetT);


    // define projection type
    DOMElement* ProjT = Document->createElement(ProjTypeXMLStr);

    if(Geometry->ProjType == PT_STRIP) {
      tmpXMLStr = XMLString::transcode("strip");
    }
    else {
      tmpXMLStr = XMLString::transcode("line");
    }

    DOMText* ProjTText = Document->createTextNode(tmpXMLStr);
    //delete tmpXMLStr;
    ProjT->appendChild(ProjTText);

    // append projection type to parallel
    Para->appendChild(ProjT);

    // append parallel to geometry
    Geom->appendChild(Para);
  }

  // append geometry to application header
  ApplicationHeaderElement->appendChild(Geom);


	// noise

  // create noise node
  DOMElement* Nois = Document->createElement(NoiseHeaderXMLStr);

	if(Noise.IsQuan) {
    DOMElement* Quan = Document->createElement(QuantumXMLStr);

    // define mean
    sprintf(valStr, "%f", Noise.QuantumMean);
    tmpXMLStr = XMLString::transcode(valStr);
    Quan->setAttribute(MeanXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // define calibration
    sprintf(valStr, "%f", Noise.QuantumCalibration);
    tmpXMLStr = XMLString::transcode(valStr);
    Quan->setAttribute(CalibMeasXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // define gantry
    sprintf(valStr, "%i", Noise.QuantumGantry);
    tmpXMLStr = XMLString::transcode(valStr);
    Quan->setAttribute(GantryXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // append quantum to noise
    Nois->appendChild(Quan);
	}


  if(Noise.IsScat) {
    DOMElement* Scat = Document->createElement(ScatterXMLStr);

    // define peak
    sprintf(valStr, "%f", Noise.ScatterPeak);
    tmpXMLStr = XMLString::transcode(valStr);
    Scat->setAttribute(PeakXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // define width
    sprintf(valStr, "%f", Noise.ScatterWidth);
    tmpXMLStr = XMLString::transcode(valStr);
    Scat->setAttribute(WidthXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // append scatter to noise
    Nois->appendChild(Scat);
	}


	if(Noise.IsAdd) {
    DOMElement* Add = Document->createElement(AdditiveXMLStr);

    // define mean
    sprintf(valStr, "%f", Noise.AdditiveMean);
    tmpXMLStr = XMLString::transcode(valStr);
    Add->setAttribute(MeanXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // define standard deviation
    sprintf(valStr, "%f", Noise.AdditiveStdDev);
    tmpXMLStr = XMLString::transcode(valStr);
    Add->setAttribute(StdDevXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // append additive to noise
    Nois->appendChild(Add);
	}
	

	if(Noise.IsMult) {
    DOMElement* Mul = Document->createElement(MultipXMLStr);

    // define mean
    sprintf(valStr, "%f", Noise.MultiplicativeMean);
    tmpXMLStr = XMLString::transcode(valStr);
    Mul->setAttribute(MeanXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // define standard deviation
    sprintf(valStr, "%f", Noise.MultiplicativeStdDev);
    tmpXMLStr = XMLString::transcode(valStr);
    Mul->setAttribute(StdDevXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // append multiplicative to noise
    Nois->appendChild(Mul);
	}

  // append noise to application header
  ApplicationHeaderElement->appendChild(Nois);

	
	// spectrum

  if(Spectrum == NULL) {
    return -2; // undefined spectrum
  }

  // create spectrum node
  DOMElement* Spec = Document->createElement(SpectrumHeaderXMLStr);

  // define number of energies
  tmpXMLStr = new XMLCh[5];
  XMLString::binToText(Spectrum->NumberOfEnergies, tmpXMLStr, 4, 10);
  Spec->setAttribute(NumOfEnerXMLStr, tmpXMLStr);
  delete tmpXMLStr;

	for(unsigned int i = 0; i < Spectrum->NumberOfEnergies; i++) {

    // create energy level node
    DOMElement* EnerL = Document->createElement(EnergyLevelXMLStr);

    // define energy
    tmpXMLStr = new XMLCh[5];
    XMLString::binToText(Spectrum->energy[i], tmpXMLStr, 4, 10);
    EnerL->setAttribute(EnergyXMLStr, tmpXMLStr);
    delete tmpXMLStr;

    // define ratio
    sprintf(valStr, "%f", Spectrum->ratio[i]);
    tmpXMLStr = XMLString::transcode(valStr);
    EnerL->setAttribute(RatioXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // define background
    sprintf(valStr, "%f", Spectrum->background[i]);
    tmpXMLStr = XMLString::transcode(valStr);
    EnerL->setAttribute(BackgroundXMLStr, tmpXMLStr);
    //delete tmpXMLStr;

    // append energy level to spectrum
    Spec->appendChild(EnerL);	
	}


  // append spectrum to application header
  ApplicationHeaderElement->appendChild(Spec);	

  // append application header to root
  RootElement->appendChild(ApplicationHeaderElement);

	return 0;
}
