/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/SARTConfig.cpp $
 $LastChangedRevision: 123 $
 $Date: 2014-07-11 09:44:57 -0400 (Fri, 11 Jul 2014) $
 $Author: bprommegger $
 ***********************************************************
 */

#include "SARTConfig.h"
#include <cstdio>
#include "uiod.h"
#include "int2str.h"
#include "infile.h"
#include "consts.h"

REAL SARTConfig::relaxiation = 1;
REAL SARTConfig::zeroThreshold = Consts.zero;

SARTConfig::SARTConfig()
{
}

SARTConfig::~SARTConfig()
{
}

void SARTConfig::printErrorMessage(char* errorString)
{
	fprintf(output, "\n          wrong configuration for SART --> default values might be applied ...");
	fprintf(output, "\n          error message: %s", errorString);
	fprintf(output, "\n              check documentation for further details !!!");
}

BOOLEAN SARTConfig::readSARTConfiguration()
{

	// definition of possible set-up parameters
	static const INTEGER strParameterRelaxation = CHAR2INT('r', 'e', 'l', 'a');
	static const INTEGER strParameterConstValue = CHAR2INT('c', 'o', 'n', 's');

	INTEGER currentParameterValueInt;
	REAL currentParameterValueReal;
	BOOLEAN eol;
	INTEGER* allowedExpressions = new INTEGER[10];
	INTEGER numberOfAllowedExpressions = 0;

	// FORMAT OF INPUT COMMAND:

	// EXECUTE SART
	// LINE OF COMMENT
	// RELAXATION CONSTANT r ... optional

	// Example:

	// EXECUTE SART
	// SART Execution Sample
	// RELAXATION CONSTANT 1.9

	// check RELAXATION
	allowedExpressions[0] = strParameterRelaxation;
	numberOfAllowedExpressions = 1;
	currentParameterValueInt = InFile.getwrd(TRUE, &eol, allowedExpressions,
			numberOfAllowedExpressions);

	// optional parameters --> EOL causes no error
	if (!eol)
	{
		switch (currentParameterValueInt)
		{

		case strParameterRelaxation:

			// check CONSTANT
			allowedExpressions[0] = strParameterConstValue;
			numberOfAllowedExpressions = 1;
			currentParameterValueInt = InFile.getwrd(FALSE, &eol,
					allowedExpressions, numberOfAllowedExpressions);

			switch (currentParameterValueInt)
			{
			case strParameterConstValue:

				// check if 0 < r <= 2
				currentParameterValueReal = InFile.getnum(FALSE, &eol);
				if (currentParameterValueReal > 0.0 && currentParameterValueReal <= 2.0)
				{
					SARTConfig::relaxiation = currentParameterValueReal;
				}
				else
				{
					SARTConfig::printErrorMessage("relaxation parameter must fit to 0.0 < r <= 2.0");
				}

				break;
			default:
				SARTConfig::printErrorMessage("second parameter must be CONS[TANT]");
				break;
			}
			break;
		default:
			SARTConfig::printErrorMessage("first parameter must be RELA[XATION]");
			break;
		}
	}

	fprintf(output, "\n          relaxation parameter is %4.3f",
			SARTConfig::relaxiation);

	delete[] allowedExpressions;

	return true;
}
