/*
 ***********************************************************
 $SNARK_Header$
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/trm6.cpp $
 $LastChangedRevision: 122 $
 $Date: 2014-07-09 18:18:59 -0400 (Wed, 09 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Residual based stopping criterion

 */

#include <cstdio>
#include <cstdlib>
#include "consts.h"
#include "infile.h"
#include "TerminationCriterion.h"

using namespace std;

TerminationCriterion::TerminationCriterion(DistanceMeasure* distanceMeasureIn,
		char* reportingFilenameIn) {
	this->terminationValue = 0;
	this->distanceMeasure = distanceMeasureIn;

	this->reportingEnabled = FALSE;
	this->reportingSkipFactor = 0;
	this->reportingFilename = reportingFilenameIn;
}

TerminationCriterion::~TerminationCriterion() {
}

void TerminationCriterion::Init() {
	// read desired epsilon from command
	BOOLEAN eol;
	this->terminationValue = InFile.getnum(FALSE, &eol);

	if (this->terminationValue < Consts_class::zero) {
		printf("\n **** epsilon must be specified and >= ZERO");
		printf("\n **** program aborted\n");
		exit(-1);
	}

	// read reporting command (disabled by default)
	INTEGER rprt = CHAR2INT('r', 'p', 'r', 't');
	if (InFile.getwrd(FALSE, &eol, &rprt, 1) == rprt) {
		this->reportingEnabled = true;
		printf("\n         reporting is enabled");
		printf("\n         reporting file: %s", this->reportingFilename);

		this->reportingSkipFactor = InFile.getnum(FALSE, &eol);
		if (this->reportingSkipFactor > 0) {
			printf("\n         report skip factor %d",
					this->reportingSkipFactor);
		} else {
			printf("\n         reporting on every iteration");
		}
	} else {
		this->reportingEnabled = false;
		printf("\n         reporting is disabled");
	}

	printf("\n         epsilon = %12.4f", this->terminationValue);
}

void TerminationCriterion::reportCurrentValue(REAL currentDistance,
INTEGER iter) {

	if (this->reportingEnabled) {

		printf("\n         current epsilon (%s) = %20.4f",
				this->distanceMeasure->getName(), currentDistance);

		FILE* RPRTfile;

		if (iter == 1) {
			RPRTfile = fopen(this->reportingFilename, "w");
			fprintf(RPRTfile, "%s stopping criterion reporting output\n",
					this->reportingFilename);
			fprintf(RPRTfile, "    iter      %s\n", this->reportingFilename);
		} else {
			RPRTfile = fopen(this->reportingFilename, "a");
		}

		fprintf(RPRTfile, " %5i  %20.9f\n", iter, currentDistance);

		fclose(RPRTfile);
	}
}

BOOLEAN TerminationCriterion::Run(REAL* recon, INTEGER* list,
REAL* weight, INTEGER iter) {

	// calculate distance
	REAL currentDistance = this->distanceMeasure->calculateDistance(recon, list,
			weight);

	if (currentDistance <= this->terminationValue) {
		this->reportCurrentValue(currentDistance, iter);

		return true;
	} else {
		if (iter == 1 || this->reportingSkipFactor == 0
				|| (iter % this->reportingSkipFactor) == 0) {
			this->reportCurrentValue(currentDistance, iter);
		}

		return false;
	}
}
