/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/bdhk_misl.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "blob.h"
#include "geom.h"
#include "fourie.h"
#include "modefl.h"
#include "consts.h"
#include "uiod.h"
#include "anglst.h"
#include "errfac.h"
#include "errpar.h"
#include "pick.h"
#include "pseudo.h"
#include <cstdio>
#include <iostream>
#include "infile.h"
#include "anglst.h"
#include <iomanip>
#include <fstream>
#include "second.h"
#include "wray.h"
#include "raysel.h"
#include "bdhk.h"
using namespace std;

REAL bdhk_class::AVER(REAL SUM, REAL NUM)
{
	return SUM / NUM;
}

REAL bdhk_class::AVER3(REAL A, REAL B, REAL C)
{
	REAL temp = (A + B + C) / 3.0;
	return temp;
}

REAL bdhk_class::diffAbs(REAL A, REAL B)
{
	REAL dif = A - B;

	if (dif < 0)
		dif *= -1;

	return dif;
}

REAL bdhk_class::neigSum(REAL* recon, REAL* d)
{
	REAL sum = 0;
	REAL Pi1, Pi2, Pi3, Pi4, Dif1, Dif2, Dif3, Dif4;
	for (int i = 0; i < GeoPar.area; i++)
	{
		if (i == 0) //upper left corner
		{
			Pi1 = AVER3(recon[1], recon[GeoPar.nelem], recon[GeoPar.nelem + 1]);
			Dif1 = diffAbs(recon[i], Pi1);
			sum += Dif1;
			d[i] = Pi1 - recon[i];
		}
		else if (i == (GeoPar.nelem - 1)) //upper right corner
		{
			Pi1 = AVER3(recon[GeoPar.nelem - 2], recon[2 * GeoPar.nelem - 2],
					recon[2 * GeoPar.nelem - 1]);
			Dif1 = diffAbs(recon[i], Pi1);
			sum += Dif1;
			d[i] = Pi1 - recon[i];
		}
		else if (i == (GeoPar.area - GeoPar.nelem)) //lower left corner
		{
			Pi1 = AVER3(recon[GeoPar.area - 2 * GeoPar.nelem],
					recon[GeoPar.area - 2 * GeoPar.nelem + 1],
					recon[GeoPar.area - GeoPar.nelem + 1]);
			Dif1 = diffAbs(recon[i], Pi1);
			sum += Dif1;
			d[i] = Pi1 - recon[i];
		}
		else if (i == (GeoPar.area - 1)) //lower right  corner
		{
			Pi1 = AVER3(recon[GeoPar.area - 2],
					recon[GeoPar.area - GeoPar.nelem - 1],
					recon[GeoPar.area - GeoPar.nelem - 2]);
			Dif1 = diffAbs(recon[i], Pi1);
			sum += Dif1;
			d[i] = Pi1 - recon[i];
		}

		else if (i < GeoPar.nelem) //first row, not corners
		{
			//two possibilities:

			Pi1 = AVER3(recon[i - 1], recon[i - 1 + GeoPar.nelem],
					recon[i + GeoPar.nelem]);
			Pi2 = AVER3(recon[i + 1], recon[i + 1 + GeoPar.nelem],
					recon[i + GeoPar.nelem]);

			Dif1 = diffAbs(recon[i], Pi1);
			Dif2 = diffAbs(recon[i], Pi2);

			if (Dif1 < Dif2)
			{
				sum += Dif1;
				d[i] = Pi1 - recon[i];

			}
			else
			{
				sum += Dif2;
				d[i] = Pi2 - recon[i];
			}
		}

		else if (i % GeoPar.nelem == 0) //first column, not corners
		{
			//two possibilities:

			Pi1 = AVER3(recon[i + 1], recon[i - GeoPar.nelem],
					recon[i - GeoPar.nelem + 1]);
			Pi2 = AVER3(recon[i + 1], recon[i + GeoPar.nelem],
					recon[i + GeoPar.nelem + 1]);

			Dif1 = diffAbs(recon[i], Pi1);
			Dif2 = diffAbs(recon[i], Pi2);

			if (Dif1 < Dif2)
			{
				sum += Dif1;
				d[i] = Pi1 - recon[i];
			}
			else
			{
				sum += Dif2;
				d[i] = Pi2 - recon[i];
			}

		}

		else if (i % GeoPar.nelem == GeoPar.nelem - 1) //last  column, not corners
		{
			//two possibilities:

			Pi1 = AVER3(recon[i - 1], recon[i - GeoPar.nelem - 1],
					recon[i - GeoPar.nelem]);
			Pi2 = AVER3(recon[i - 1], recon[i + GeoPar.nelem],
					recon[i + GeoPar.nelem - 1]);

			Dif1 = diffAbs(recon[i], Pi1);
			Dif2 = diffAbs(recon[i], Pi2);

			if (Dif1 < Dif2)
			{
				sum += Dif1;
				d[i] = Pi1 - recon[i];
			}
			else
			{
				sum += Dif2;
				d[i] = Pi2 - recon[i];
			}

		}
		else if (i > GeoPar.area - GeoPar.nelem) //last  row, not corners
		{
			//two possibilities:

			Pi1 = AVER3(recon[i - 1], recon[i - GeoPar.nelem - 1],
					recon[i - GeoPar.nelem]);
			Pi2 = AVER3(recon[i + 1], recon[i - GeoPar.nelem],
					recon[i - GeoPar.nelem + 1]);

			Dif1 = diffAbs(recon[i], Pi1);
			Dif2 = diffAbs(recon[i], Pi2);

			if (Dif1 < Dif2)
			{
				sum += Dif1;
				d[i] = Pi1 - recon[i];
			}
			else
			{
				sum += Dif2;
				d[i] = Pi2 - recon[i];
			}

		}
		else //not in the bounderies but in the middle
		{
			//four  possibilities:

			Pi1 = AVER3(recon[i - 1], recon[i - GeoPar.nelem - 1],
					recon[i - GeoPar.nelem]);
			Pi2 = AVER3(recon[i + 1], recon[i - GeoPar.nelem],
					recon[i - GeoPar.nelem + 1]);
			Pi3 = AVER3(recon[i - 1], recon[i - 1 + GeoPar.nelem],
					recon[i + GeoPar.nelem]);
			Pi4 = AVER3(recon[i + 1], recon[i + GeoPar.nelem],
					recon[i + GeoPar.nelem + 1]);

			Dif1 = diffAbs(recon[i], Pi1);
			Dif2 = diffAbs(recon[i], Pi2);
			Dif3 = diffAbs(recon[i], Pi3);
			Dif4 = diffAbs(recon[i], Pi4);

			if (Dif1 <= Dif2 && Dif1 <= Dif3 && Dif1 <= Dif4)
			{
				sum += Dif1;
				d[i] = Pi1 - recon[i];
			}
			else if (Dif2 <= Dif1 && Dif2 <= Dif3 && Dif2 <= Dif4)
			{
				sum += Dif2;
				d[i] = Pi2 - recon[i];
			}
			else if (Dif3 <= Dif1 && Dif3 <= Dif2 && Dif3 <= Dif4)
			{
				sum += Dif3;
				d[i] = Pi3 - recon[i];
			}
			else if (Dif4 <= Dif1 && Dif4 <= Dif2 && Dif4 <= Dif3)
			{
				sum += Dif4;
				d[i] = Pi4 - recon[i];
			}
		}

	}
	REAL norm_d = calcNorm(d);

	if (norm_d != 0)
	{
		//normalizing the d vector:
		//~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		for (int i1 = 0; i1 < GeoPar.area; i1++)
			d[i1] /= norm_d;
	}

	return sum;
}

REAL bdhk_class::calculateResidual(REAL* recon, INTEGER* list, REAL* weight,
		struct NPNR* arrayRays, INTEGER kount)
{
	INTEGER np, nr, i, j, numb;
	REAL snorm, temp1, b, res = 0;
	for (i = 0; i < kount; i++)
	{
		np = arrayRays[i].NP;
		nr = arrayRays[i].NR;
		wray(np, nr, list, weight, &numb, &snorm);
		b = Anglst.prdta(np, nr);
		if (numb != 0)
		{
			temp1 = 0;
			for (j = 0; j < numb; j++)
				temp1 += weight[j] * recon[list[j]];
			res += (pow((b - temp1), 2)) / snorm;
		}	//end if
	}	//end for
	res = sqrt(res);
	return res;
}

BOOLEAN bdhk_class::calculateGradientTV(REAL* grad, REAL* recon)
{
	//initialiaze the Gradient vector to all zero
	for (int iter1 = 0; iter1 < GeoPar.area; iter1++)
		grad[iter1] = 0;

	//looping through case 1
	for (int k = 0; k < (GeoPar.area - GeoPar.nelem); k++) //not including the last row
	{
		if (((k % GeoPar.nelem) - (GeoPar.nelem - 1)) != 0) //not in the last column
		{

			REAL v1 = recon[k + 1] - recon[k];
			REAL v2 = recon[k + GeoPar.nelem] - recon[k];
			REAL deno = sqrt(v1 * v1 + v2 * v2);
			if (deno >= Consts.zero) //continue calculating the subgradients
				//there are three subgradients to consider. They are placed in k, k+1, k+nelem
				// (1) 2x_k - x_k+1 - x_k+nelem / deno
				grad[k] += (2 * recon[k] - recon[k + 1]
						- recon[k + GeoPar.nelem]) / deno;

		}        //end if

	}        //end for k

			 //looping through case 2
	for (int k = 0; k < (GeoPar.area - GeoPar.nelem); k++) //not including the last row
	{
		if ((k % GeoPar.nelem) != 0) //not in the first column
		{

			REAL v1 = recon[k] - recon[k - 1];
			REAL v2 = recon[k + GeoPar.nelem - 1] - recon[k - 1];
			REAL deno = sqrt(v1 * v1 + v2 * v2);
			if (deno >= Consts.zero) //continue calculating the subgradients
				grad[k] += (recon[k] - recon[k - 1]) / deno;

		} //end if

	} //end for k

	  //looping through case 3
	for (int k = GeoPar.nelem; k < (GeoPar.area); k++) //not including the first row
	{
		if (((k % GeoPar.nelem) - (GeoPar.nelem - 1)) != 0) //not in the last column
		{

			REAL v1 = recon[k + 1 - GeoPar.nelem] - recon[k - GeoPar.nelem];
			REAL v2 = recon[k] - recon[k - GeoPar.nelem];
			REAL deno = sqrt(v1 * v1 + v2 * v2);
			if (deno >= Consts.zero) //continue calculating the subgradients
				grad[k] += (recon[k] - recon[k - GeoPar.nelem]) / deno;

		} //end if

	} //end for k

	  // BOOLEAN flag=FALSE;
	for (int i = 0; i < GeoPar.area; i++)
	{
		if (grad[i] != 0)
			return TRUE;
	} //end for

	return FALSE;

} //end calculateGradient

void bdhk_class::orderRays(struct NPNR* arrayRays, INTEGER kount)
{
	INTEGER np, nr, i;
	for (i = 0; i < kount; i++)
	{
		pick(&np, &nr);
		struct NPNR temp;
		temp.NP = np;
		temp.NR = nr;
		arrayRays[i] = temp;
	}
}

void bdhk_class::MyART(REAL* recon, INTEGER* list, REAL* weight,
		struct NPNR* arrayRays, INTEGER kount)
{
	INTEGER np, nr, i, j, numb;
	REAL snorm, temp, b, div, dif;

	for (i = 0; i < kount; i++)
	{
		np = arrayRays[i].NP;
		nr = arrayRays[i].NR;
		wray(np, nr, list, weight, &numb, &snorm);
		if (numb != 0)
		{
			if (snorm <= Consts.zero)
			{
				cout << endl << "*** error ---> norm square is less than "
						<< Consts.zero << " ***";
				cout << endl << " *** ART CALCULATION ABORTED ***" << endl;
				return;
			}
			temp = 0;
			for (j = 0; j < numb; j++)
				temp += weight[j] * recon[list[j]];
			b = Anglst.prdta(np, nr);
			dif = b - temp;
			div = dif / snorm;
			for (j = 0; j < numb; j++)
				recon[list[j]] += div * weight[j];

		} //end if
	} //end for
	return;
} //end MyART

REAL bdhk_class::calcHalfNormSquare(REAL* recon)
{
	REAL sum = squareNorm(recon) / 2;
	return sum;
}

REAL bdhk_class::calcTV(REAL* recon)
{
	REAL sum = 0.0;

	for (int x = 0; x < (GeoPar.area - GeoPar.nelem - 1); x++)
	{
		if (((x % GeoPar.nelem) - (GeoPar.nelem - 1)) != 0) //not last column
			sum += sqrt(
					pow((recon[x + 1] - recon[x]), 2)
							+ pow((recon[x + GeoPar.nelem] - recon[x]), 2));

	}

	return sum;
}

REAL bdhk_class::squareNorm(REAL* aVector)
{
	REAL sum = 0;
	for (int i = 0; i < GeoPar.area; i++)
		sum += aVector[i] * aVector[i];
	return sum;
}

REAL bdhk_class::calcNorm(REAL* aVector)
{
	return sqrt(squareNorm(aVector));
}

