/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/blob_footp.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 subroutine to perform the forward (1) or backward (-1) "projection" using blob footprints

 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "blob.h"
#include "geom.h"
#include "consts.h"
#include "uiod.h"

void Blob_class::footp(REAL *blob_array, REAL *pict, INTEGER option)
{
	INTEGER mult; //max number of pixels whose center is covered by a blob
	INTEGER m, h, m1, h1; // (m,h) coord of the center of a blob in the underlying sq grid. m1 * Blob.H + h1 index in the blob array.
	INTEGER k, j; //(j,k) coord of the center of a pixel
	INTEGER k0, j0;
	INTEGER jc, kc; // index of the nearest pixel
	INTEGER jf, kf, jl, kl; //upper and lower bound for j and k given a blob
	INTEGER nelem2;
	INTEGER jx, ky;
	INTEGER ind, n;

	REAL xc, yc, auy;
	REAL ipixsiz = (REAL) 1.0 / GeoPar.pixsiz;
	REAL dist, distx, disty; // dist is the distance betw a blob center and a pixel center. (distx, disty) is the vector distance
	REAL dispx, dispy; // (dispx, dispy) is the displacement from the center of a blob to the center of the nearest pixel.

	mult = (INTEGER) floor(Blob.support / GeoPar.pixsiz + 0.5) + 1;

	nelem2 = (INTEGER) (GeoPar.nelem / 2);

	REAL* footx = new REAL[2 * mult + 1];
	REAL* footy = new REAL[2 * mult + 1];

	for (k0 = -mult; k0 <= mult; k0++)
	{
		footy[k0 + mult] = (REAL) k0 * GeoPar.pixsiz;
	}

	for (j0 = -mult; j0 <= mult; j0++)
	{
		footx[j0 + mult] = (REAL) j0 * GeoPar.pixsiz;
	}

	// initialization for forward operation
	if (option == 1)
	{
		for (k = 0; k < GeoPar.nelem; k++)
		{
			for (j = 0; j < GeoPar.nelem; j++)
			{
				pict[k * GeoPar.nelem + j] = 0.0;
			}
		}
	}

	//initialization for backward operation
	else if (option == -1)
	{
		for (j = 0; j <= Blob.area; j++)
		{
			if (j % 2 == Blob.pr)
			{
				blob_array[j] = 0.0;
			}
		}
	}

	// forward or backward operation

	for (h = lhf; h <= lhl; h++)
	{
		for (m = lmf; m <= lml; m++)
		{ // m and h must have the same parity!!
			if ((m + h) % 2 == 0)
			{

				xc = di2 * m;  // coord of the center of the blob (m,h)
				yc = di2sq3 * h;
				auy = ipixsiz * yc;
				jx = (INTEGER) floor(ipixsiz * xc + 0.5); // coord of the center of

				ky = (INTEGER) floor(auy + 0.5);        // the nearest pixel

				jc = jx + nelem2; // bring to the right address
				kc = (INTEGER) floor(-auy + 0.5) + nelem2;

				// displacement of the footprint with resp to the nearest pixel center
				dispx = jx * GeoPar.pixsiz - xc;

				dispy = ky * GeoPar.pixsiz - yc;

				jf = MAX0(ljf, jc - mult);
				jl = MIN0(ljl, jc + mult);

				kf = MAX0(lkf, kc - mult);
				kl = MIN0(lkl, kc + mult);

				if (jl < ljf || jf > ljl || kl < lkf || kf > lkl)
					continue;

				m1 = m + Blob.M2;
				h1 = h + Blob.H2;

				ind = h1 * Blob.M + m1;

				for (k = kf; k <= kl; k++)
				{
					k0 = k - kc;
					disty = footy[k0 + mult] - dispy; //the y in the x-y coord and the k in the j-k coord are in the opposite directions
					if (fabs(disty) > Blob.support)
						continue;

					for (j = jf; j <= jl; j++)
					{
						j0 = j - jc;
						distx = footx[j0 + mult] + dispx;
						if (fabs(distx) > Blob.support)
							continue;

						dist = sqrt(distx * distx + disty * disty);
						if (dist < Blob.support)
						{
							n = (INTEGER) round(dist * sampiblob); // changed "rint" to "round". Lajos, Jan 25, 2005
							//forward case
							if (option == 1)
							{
								pict[k * GeoPar.nelem + j] += (blob_array[ind]
										* Blob.vtable[n]);
							}

							//backward case
							else if (option == -1)
							{
								blob_array[ind] += (pict[k * GeoPar.nelem + j]
										* Blob.vtable[n]);
							}
						}

					}
				}

			}
		}
	}

	delete[] footx;  // bug 92 - Lajos - 03/02/2005
	delete[] footy;  // bug 92 - Lajos - 03/02/2005
	return;
}

