/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/fft2p.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 fft2p.cpp,v 1.2 2008/08/25 16:11:08 jklukowska Exp

 THE SUBROUTINES FFT2P , FTODD , UNKPS ARE FAST FOUTIER ROUTINES.
 THE SUBROUTINE SNFFT ,DESCRIBED IN THE SNARK05 MANUAL , MAKES
 USE OF THESE SUBROUTINES.
 THE SUBROUTINES HAVE BEEN TAKEN FROM
 POLGE,R.J., AND BHAGAVAN,B.K., EFFICIENT FAST FOURIER TRANSFORM
 PROGRAMS FOR ARBITRARY FACTORS WITH ONE STEP LOOP UNSCRAMBLING,
 IEEE TRANSACTIONS ON COMPUTERS, MAY 1976.
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "ftodd.h"
#include "unkps.h"
#include "consts.h"
#include "fft2p.h"

void fft2p(
INTEGER imax,
INTEGER istep,
REAL* xs,
REAL* x,
INTEGER m,
INTEGER invdir,
INTEGER* ix,
INTEGER nipf,
INTEGER* ipf,
INTEGER* ipu,
REAL* vspt,
REAL* t,
BOOLEAN virt)
{

	REAL v[2], vg[2];

	//BOOLEAN dovirt;
	INTEGER sk, ipfx[8];

	REAL tpis;
	INTEGER m2;
	INTEGER mc;
	INTEGER irp;
	INTEGER k;
	INTEGER nfixp;
	INTEGER nipfx;
	INTEGER ir;
	INTEGER n;
	INTEGER k4m4;
	INTEGER m1;
	INTEGER mx;
	REAL fi;
	INTEGER nk;
	INTEGER j;
	INTEGER kp;
	INTEGER ia;
	INTEGER ib;
	INTEGER l;
	INTEGER index1;
	INTEGER index2;
	REAL temp;

	//dovirt = virt && (istep > 1);

	tpis = ((REAL) 2.0) * Consts.pi / ((REAL) invdir);
	m2 = m / 2;
	mc = m - 2 * m2;
	irp = 1;

	if (nipf != 0)
	{
		for (k = 0; k < nipf; k++)
		{
			irp *= ipf[k];
		}
		nfixp = 0;
	}

	nipfx = nipf + mc;
	ir = irp * (1 << mc);
	n = (1 << (m - mc)) * ir;

	k4m4 = 1 << m2;

	m1 = m2 + mc;
	mx = m;
	fi = tpis / n;

	v[0] = (REAL) cos(fi);
	v[1] = (REAL) sin(fi);

	nk = n;
	if (mx != 0)
	{

		L1: sk = nk / 2;
		vg[0] = 1.0;
		vg[1] = 0.0;
		for (j = 0; j < sk; j++)
		{
			for (k = j; k < n; k += nk)
			{
				kp = k + sk;
				ia = k * istep;

				ib = kp * istep;

				for (l = 0; l < imax; l++)
				{

					index1 = l + ia;
					index2 = l + ib;

					CA(xs, 0, 0)= CA(x, 0, index1);
					CA(xs, 1, 0)= CA(x, 1, index1);

					CA(x, 0, index1)+= CA(x, 0, index2);
					CA(x, 1, index1)+= CA(x, 1, index2);

					CA(xs, 0, 0)-= CA(x, 0, index2);
					CA(xs, 1, 0)-= CA(x, 1, index2);

					CA(x, 0, index2)= vg[0] * CA(xs, 0, 0) - vg[1] * CA(xs, 1, 0);
					CA(x, 1, index2)= vg[0] * CA(xs, 1, 0) + vg[1] * CA(xs, 0, 0);
				}
			}
			temp = vg[0] * v[0] - vg[1] * v[1];
			vg[1] = vg[0] * v[1] + vg[1] * v[0];
			vg[0] = temp;
		}
		nk = sk;
		temp = (v[0] - v[1]) * (v[0] + v[1]);
		v[1] = ((REAL) 2.0) * v[0] * v[1];
		v[0] = temp;
		m1--;
		mx--;
		if (m1 > 0)
			goto L1
			;
		if (mx != 0)
		{
			m1 = m2;
		}
	}

	if (irp != 1)
	{

		ftodd(imax, istep, xs, x, n, &nk, v, tpis, nipf, ipf, mc, k4m4, nipfx, ipfx, ipu, ix, &nfixp, vspt, t, virt);

		irp = 1;
	}

	if (m1 != 0)
		goto L1;

	unkps(imax, istep, xs, x, m, nipfx, 2, mc, ir, n, ix, ipu, nfixp, k4m4, virt);
	return;
}
