/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/fom_klds.c $
 $LastChangedRevision: 137 $
 $Date: 2014-07-15 16:38:33 -0400 (Tue, 15 Jul 2014) $
 $Author: bprommegger $
 ***********************************************************
 */

// Extracts the Kullback-Leibler distance as a fom
#include <assert.h>
#include "malloc.h"
#include "experimenter.h"
#include "read_eval_recon4.h"

/*
 This function reads the kullback-leibler distance.

 INPUTS:
 itr1 - array of length niters containing the iteration numbers for the first algorithm
 itr2 - array of length niters containing the iteration numbers for the second algorithm
 niters - number of iterations to be read
 keywrd1 - string containing the keyword which defines the first algorithm
 keywrd2 - string containing the keyword which defines the second algorithm

 OUTPUTS:
 klds1 - array of length niters containing the kullback-leibler distances for the first algorithm
 klds2 - array of length niters containing the kullback-leibler distances for the second algorithm
 */
void fom_klds(int* itr1, int* itr2, int niters, char* keywrd1, char* keywrd2, double* klds1, double* klds2)
{
	int i = 0;
	double dummy_wsqd = 0.0;
	for (i = 0; i < niters; i++)
	{
		read_eval_recon4(itr1[i], keywrd1, &klds1[i], &dummy_wsqd);
		read_eval_recon4(itr2[i], keywrd2, &klds2[i], &dummy_wsqd);
	}
}
