/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/foru_uint.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************


 PURPOSE: GIVEN THE COORDINATES OF THE DESIRED POINT  (IX,IY)
 AND THE TWO TRANSFORMED PROJECTIONS  PJ1 AND PJ2 FIND THE
 INTERPOLATED VALUE (GR,GI) THEN CALL THE SUBROUTINE STORE
 TO STORE THE VALUE IN APPROPRIATE PLACE IN THE ARRAY
 STARTING AT NFRPLN+1
 THE INTERPOLATION SCHEME TO BE USED IS DETERMINED BY
 INTERP (SEE ALSO COMMENTS IN THE MAIN PROGRAM )
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "fourie.h"
#include "consts.h"
#include "uiod.h"

#include "foru.h"

#define disn(b,c,the)  ((REAL) sqrt((b)*(b)+(c)*(c)-2.*(b)*(c) * (REAL) cos(the)))

void foru_class::uint1(INTEGER icase, INTEGER ix, INTEGER iy, REAL ang1, REAL ang2, REAL* pj1, REAL* pj2)
{

	INTEGER x;
	INTEGER y;
	REAL r;
	REAL ang;
	REAL a;
	INTEGER low;
	INTEGER indmin;
	INTEGER indmax;
	REAL s;
	REAL t;
	INTEGER ind;
	REAL gr;
	REAL gi;
	REAL pr;
	REAL qr;
	REAL px;
	REAL qi;
	REAL ak;
	REAL d1;
	REAL d2;
	REAL d3;
	REAL d4;
	REAL e1;
	REAL e2;
	REAL e3;
	REAL e4;
	REAL den;

	x = ix;
	y = iy;
	r = (REAL) sqrt((REAL) x * x + y * y) * Fourie.delp;
	ang = (REAL) atan2((REAL) y, (REAL) x);
	a = r / Fourie.deld;
	if (a >= Fourie.radmax)
		return;
	low = (INTEGER) a;
	indmin = 2 * low;
	indmax = indmin + 2;
	s = (REAL) fabs((ang - ang1) / (ang2 - ang1));
	t = a - low;

	switch (Fourie.interp)
	{

	// INTERP = 1 NEAREST NEIGHBOR IN CARTESIAN SENSE
	case 1:
		if (s <= 0.5)
		{
			ind = indmax;
			if ((a * cos(ang - ang1) - low) <= 0.5)
				ind = indmin;
			gr = pj1[ind];
			gi = pj1[ind + 1];
		}
		else
		{
			ind = indmax;
			if ((a * cos(ang2 - ang) - low) <= 0.5)
				ind = indmin;
			gr = pj2[ind];
			gi = pj2[ind + 1];
		}
		break;

		// INTERP = 2 NEAREST NEIGHBOR IN RADIAL SENSE
	case 2:
		if (t <= 0.5)
			ind = indmin;
		if (t > 0.5)
			ind = indmax;
		if (s <= 0.5)
		{
			gr = pj1[ind];
			gi = pj1[ind + 1];
		}
		else
		{
			gr = pj2[ind];
			gi = pj2[ind + 1];
		}
		break;

		// INTERP = 3 LINEAR INTERPOLATION
	case 3:
		if (t <= Consts.zero)
			t = 0.0;
		if ((1.0 - t) <= Consts.zero)
			t = 1.0;
		if (s <= Consts.zero)
			s = 0.0;
		if ((1.0 - s) <= Consts.zero)
			s = (REAL) 1.0;
		pr = t * pj1[indmax] + ((REAL) 1.0 - t) * pj1[indmin];
		qr = t * pj2[indmax] + ((REAL) 1.0 - t) * pj2[indmin];
		gr = pr * ((REAL) 1.0 - s) + qr * s;
		px = t * pj1[indmax + 1] + ((REAL) 1.0 - t) * pj1[indmin + 1];
		qi = t * pj2[indmax + 1] + ((REAL) 1.0 - t) * pj2[indmin + 1];
		gi = px * ((REAL) 1.0 - s) + qi * s;
		break;

		// INTERP = 4 F=(F1/D1+F2/D2+F3/D3+F4/D4)/(1/D1+1/D2+1/D3+1/D4)
	case 4:
		ak = low;
		d1 = disn(a, ak, ang - ang1);
		d2 = disn(a, ak + 1, ang - ang1);
		d3 = disn(a, ak, ang2 - ang);
		d4 = disn(a, ak + 1, ang2 - ang);
		e1 = d2 * d3 * d4;
		e2 = d1 * d3 * d4;
		e3 = d1 * d2 * d4;
		e4 = d1 * d2 * d3;


		if (e1 <= Consts.zero)
			e1 = 0;
		if (e2 <= Consts.zero)
			e2 = 0;
		if (e3 <= Consts.zero)
			e3 = 0;
		if (e4 <= Consts.zero)
			e4 = 0;

		den = e1 + e2 + e3 + e4;


		gr = (pj1[indmin] * e1 + pj1[indmax] * e2 + pj2[indmin] * e3
				+ pj2[indmax] * e4) / den;

		gi = (pj1[indmin + 1] * e1 + pj1[indmax + 1] * e2 + pj2[indmin + 1] * e3
				+ pj2[indmax + 1] * e4) / den;
	}

	// CALL THE SUBROUTINE STORE TO STORE THE VALUE (GR,GI) AT POINT
	// (IX,IY) IN APPROPRIATE PLACE IN THE ARRAY STARTING AT NFRPLN+1

	store(ix, iy, gr, gi);
	return;
}
