/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/foru_vc56.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 VC56: VARIABLE SPACING CASE 5 AND 6
 PURPOSE: FOR VARIABLE SPACING, ICASE=5 OR 6 (THE TWO LINES LIE
 BETWEEN 45 AND 90 DEGREE LINES OR ANG1 IS IN 1ST QUADRANT
 AND ANG2 IS IN 4TH QUADRANT) FIND ALL POINTS THAT LIE
 BETWEEN THE TWO LINES AND INTERPOLATE
 PJ1 AND PJ2 ARE THE ARRAYS  HOLDING THE TRANSFORMED PROJS
 ANG1 AND ANG2 ARE THE CORRESPONDING ANGLES
 FOR CASE 6 THE SECOND ANGLE, ANG2, WILL BE ADDED PI TO
 BECOME AN ANGLE IN SECOND QUADRANT TO PERFORM
 INTERPOLATION
 */

// bug79 - problem with fourier algorithm
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "fourie.h"
#include "consts.h"

#include "foru.h"

#define  cot(z)  (cos(z)/sin(z));

void foru_class::vc56(INTEGER icase, REAL ang1, REAL ang2, REAL* pj1, REAL* pj2)
{
	REAL xx[4];
	REAL yy[4];

	INTEGER khd;
	INTEGER kk;
	REAL uleft;
	REAL uright;
	INTEGER kh;
	INTEGER j;
	INTEGER iy;
	REAL aleft;
	REAL aright;
	INTEGER indl;
	INTEGER indh;
	REAL a;
	INTEGER low;
	INTEGER indmin;
	INTEGER indmax;
	REAL distan;
	REAL yg;
	INTEGER ix;
	REAL xg;
	REAL gr;
	REAL gi;
	REAL amid;
	INTEGER ind;
	INTEGER x;
	REAL t;
	REAL pr;
	REAL qr;
	REAL px;
	REAL qi;
	REAL s;

	///cot(z)=cos(z)/sin(z)

	if (icase != 5)
	{

		khd = Fourie.nsize1 + 2;

		for (kk = 1; kk < khd; kk += 2)
		{     // bug79 - swr 2/11/05
			pj2[kk] = -pj2[kk];
		}

		ang2 += Consts.pi;
	}

	uleft = (REAL) cot(ang2)
	;
	uright = (REAL) cot(ang1)
	;
	kh = Fourie.nsize2 / 2 - 1;

	for (j = 1; j <= kh; j++)
	{     // bug79 - swr 2/11/05

		iy = j;
		aleft = uleft * iy;
		aright = uright * iy;
		indl = (INTEGER) ceil(aleft - Fourie.epsln); // bug82 - changed to ceil - swr - 2/20/05
		indh = (INTEGER) floor(aright + Fourie.epsln); // bug82 - changed to floor - swr - 2/20/05

		if (indl <= indh)
		{
			a = j * Fourie.delp / Fourie.deld;
			low = (INTEGER) a;
			indmin = 2 * low;     // bug79 - swr 2/11/05
			indmax = indmin + 2;
			distan = aright - aleft;

			switch (Fourie.interp)
			{
			// INTERP = 1 NEAREST NEIGHBOR IN CARTESIAN SENSE
			// INTERP = 4 F=(F1/D1+F2/D2+F3/D3+F4/D4)/(1/D1+1/D2+1/D3+1/D4)
			case 1:
			case 4:
				yy[1] = (REAL) low;
				yy[3] = (REAL) low;
				yy[2] = (REAL) low + 1;
				yy[4] = (REAL) low + 1;
				xx[1] = yy[1] * uright;
				xx[2] = yy[2] * uright;
				xx[3] = yy[3] * uleft;
				xx[4] = yy[4] * uleft;
				yg = a;

				for (ix = indl; ix <= indh; ix++)
				{     // bug79 - swr 2/11/05
					xg = ix * Fourie.delp / Fourie.deld;
					vint14(xx, yy, xg, yg, pj1, pj2, indmin, indmax, &gr, &gi); // changed to pass address of "gr" and "gi". Lajos, Feb 10, 2005
					store(ix, iy, gr, gi);
				}
				break;

				// INTERP = 2 NEAREST NEIGHBOR IN RADIAL SENSE
			case 2:
				amid = (aleft + aright) / (REAL) 2.0;
				ind = 2 * (lround(a)) + 1; // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
				for (ix = indl; ix <= indh; ix++)
				{     // bug79 - swr 2/11/05
					x = ix;
					if (x <= amid)
					{     // bug79 - swr 2/15/05
						gr = pj2[ind - 1];
						gi = pj2[ind];
					}
					else
					{
						gr = pj1[ind - 1];
						gi = pj1[ind];
					}
					store(ix, iy, gr, gi);
				}
				break;

				// INTERP = 3 LINEAR INTERPOLATION
			case 3:
				t = a - low;
				if (t <= Consts.zero)
					t = 0.0;
				if ((1. - t) <= Consts.zero)
					t = (REAL) 1.0;
				pr = pj1[indmin] * ((REAL) 1.0 - t) + pj1[indmax] * t;
				qr = pj2[indmin] * ((REAL) 1.0 - t) + pj2[indmax] * t;
				px = pj1[indmin + 1] * ((REAL) 1.0 - t) + pj1[indmax + 1] * t;
				qi = pj2[indmin + 1] * ((REAL) 1.0 - t) + pj2[indmax + 1] * t;

				for (ix = indl; ix <= indh; ix++)
				{     // bug79 - swr 2/11/05
					s = (aright - ix) / distan;
					if (s <= Consts.zero)
						s = 0.0;
					if ((1. - s) <= Consts.zero)
						s = 1.0;
					gr = pr * ((REAL) 1.0 - s) + qr * s;
					gi = px * ((REAL) 1.0 - s) + qi * s;
					store(ix, iy, gr, gi);
				}
				break;
			}
		}
	}

	if (icase == 5)
		return;

	khd = Fourie.nsize1 + 2;
	for (kk = 1; kk < khd; kk += 2)
	{     // bug79 - swr 2/11/05
		pj2[kk] = -pj2[kk];
	}
	ang2 -= Consts.pi;
	return;
}
