/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/getden.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 ---  getden.f

 ---  Subroutine to get thresholding density values t1 (lower limit) and t2
 (upper limit) if valid density selection is specified in input,
 otherwise set t1 = minus infinit    ; t2 = plus infinity
 */

//      parameter (MXNREG = 40)

#include <cstdio>
#include <cstdlib>

#include "blkdta.h"
#include "consts.h"
#include "uiod.h"
#include "infile.h"

#include "eval.h"

void Eval_class::getden(BOOLEAN* flagdw, INTEGER ObjJ)
{
	BOOLEAN eol;

	REAL v1 = InFile.getnum(FALSE, &eol);
	REAL v2 = InFile.getnum(FALSE, &eol);

	// Set density range ifspecification is valid, otherwise set default
	// density range from minus infinity to plus infinity

	if (eol || (v1 > v2))
	{

		Obj[ObjJ].t1 = -Consts.infin;
		Obj[ObjJ].t2 = Consts.infin;
		*flagdw = FALSE;

		if (v1 > v2)
		{
			// ERROR - lower threshold is greater than upper threshold
			fprintf(output,
					"\n **** lower density value is greater than the higher");
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		}
	}
	else
	{
		Obj[ObjJ].t1 = v1;
		Obj[ObjJ].t2 = v2;
		*flagdw = TRUE;
	}
}

void Eval_class::getdens(BOOLEAN* flagdw, eval_Object_Node *o_node)
{
	BOOLEAN eol;

	// Set density range ifspecification is valid, otherwise set default
	// density range from minus infinity to plus infinity

	REAL v1 = InFile.getnum(FALSE, &eol);
	if (eol)
	{
		o_node->data.t1 = -Consts.infin;
		o_node->data.t2 = Consts.infin;
		*flagdw = FALSE;
		return;
	};

	REAL v2 = InFile.getnum(FALSE, &eol);
	if (eol)
	{
		o_node->data.t1 = -Consts.infin;
		o_node->data.t2 = Consts.infin;
		*flagdw = FALSE;
		return;
	};

	if (v1 > v2)
	{
		// ERROR - lower threshold is greater than upper threshold
		fprintf(output,
				"\n **** lower density value is greater than the higher");
		fprintf(output, "\n **** program aborted\n");
		exit(-1);
	}
	else
	{
		o_node->data.t1 = v1;
		o_node->data.t2 = v2;
		*flagdw = TRUE;
	}
}
