/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/halft.cpp $
 $LastChangedRevision: 118 $
 $Date: 2014-07-09 14:22:29 -0400 (Wed, 09 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 POST-PROCESSING ROUTINE TO GENERATE PSEUDO-HALF-TONE DISPLAYS OF
 RECONSTRUCTIONS ON THE LINE PRINTER.
 */

#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <cmath>

#include "blkdta.h"

#include "uiod.h"
#include "int2str.h"

#include "recfile.h"
#include "infile.h"

#include "modefl.h"
#include "halft.h"

void strtrim(char* out, const char* in)
{
	unsigned int first = 0;
	//find first non white from the left
	while ((in[first] == ' ') || (in[first] == '\t'))
	{  // removed third term of the condition. lajos, Nov 18, 2004
		first++;
	}

	unsigned int last = strlen(in) - 1;
	//find first non white from the right
	while (last > first)
	{
		if ((in[last] != ' ') && (in[last] != '\t'))
		{
			break;
		}
		last--;
	}

	unsigned int i;

	for (i = 0; i < last - first + 1; i++)
	{
		out[i] = in[first + i];
	}

	out[i] = 0;
}

int SavePGM(char* filename, unsigned char* array, int xdim, int ydim,
		char* coment)
{
	FILE* fpout;

	if ((fpout = fopen(filename, "w")) == NULL)
	{
		return -1;
	}


	fprintf(fpout, "P5\n");
	fprintf(fpout, "#%s\n", coment);
	fprintf(fpout, "%d %d\n", xdim, ydim);
	fprintf(fpout, "255\n");
	fwrite(array, xdim * ydim, sizeof(unsigned char), fpout);

	fclose(fpout);

	return 0;
}

void SaveImage(
REAL* x,
REAL amax,
REAL amin,
INTEGER nrow,
INTEGER mcol,
INTEGER nran,
INTEGER mp,
CHAR* filename,
CHAR* coment)
{
	int i;

	INTEGER np;
	REAL range;
	REAL ot;

	// allocate space for image buffer
	unsigned char* array = new unsigned char[nrow * mcol];
	// changed "CHAR" to "char". lajos Nov 17, 2004

	if (array == NULL)
	{
		fprintf(output, "\n **** memory allocation error");
		fprintf(output, "\n **** program aborted\n");
		exit(-1); // Error
	}

	// FORM INTENSITY VALUES MEAN AND DIFFERENCE ARRAYS

	np = mp;

	range = amax - amin;

	if (np == 2)
	{
		fprintf(output, "\n          intensity levels printed\n");
	}
	else
	{
		np = 1;
		fprintf(output, "\n          amplitude levels printed\n");
	}

	range = (REAL) 1.0 / range;

	fprintf(output, "\n          columns %5i to %5i being printed\n", 1, mcol);

	for (i = 0; i < nrow * mcol; i++)
	{

		// AMPLITUDE OR INTENSITY

		ot = MAX0((REAL) 0.0, MIN0((REAL) 1.0, ((x[i] - amin) * range)));

		if (np == 2)
			ot = (REAL) sqrt(ot);

		array[i] = (unsigned char) (255 * ot);
		// changed "(CHAR)" to "(unsigned char)". lajos Nov 18, 2004

	}

	SavePGM(filename, array, mcol, nrow, coment);

	delete[] array;  // bug 92 - Lajos - 03/02/2005

	return;
}

void halft()
{
	// was in post but it is realy local
	CHAR prjnam[81];
	CHAR phnnam[81];
	CHAR recnam[81];

	// was in geo but it is realy local
	INTEGER nelem;
	INTEGER area;
	//REAL    pixsiz;

	// bug 124 - added minimum and maximum - swr - 7/6/05
	static const INTEGER half_codes[] =
	{
			CHAR2INT('p', 'h', 'a', 'n'), CHAR2INT('d', 'i', 'f', 'f'),
			CHAR2INT('m', 'i', 'n', 'i'), CHAR2INT('m', 'a', 'x', 'i'),
			CHAR2INT('a', 'm', 'p', 'l'), CHAR2INT('i', 'n', 't', 'e') };

	int i;

	INTEGER mp;
	INTEGER nran;
	REAL amin;
	REAL amax;

	INTEGER word;
	INTEGER type;
	REAL* Phantom;
	REAL* Recon;
	INTEGER flag[51];
	CHAR algn[5];
	unsigned INTEGER count;
	unsigned INTEGER iter;
	BOOLEAN eol;
	BOOLEAN phan;
	BOOLEAN minfl;
	BOOLEAN maxfl;

	CHAR coment[200];
	CHAR FileName[100];

	type = 0;
	phan = FALSE;
	minfl = FALSE;
	maxfl = FALSE;
	mp = 1;
	nran = -1;

	// read input line
	// bug 124 - rewrote input line processing - swr - 7/6/05
	int pos = 0;
	int length = sizeof(half_codes) / sizeof(half_codes[0]);
	word = InFile.getwrd(FALSE, &eol, &half_codes[pos], length - pos);
	while (!eol)
	{
		switch (word)
		{
		case CHAR2INT('p', 'h', 'a', 'n'):
			phan = TRUE;
			pos = 2;
			break;
		case CHAR2INT('d', 'i', 'f', 'f'):
			type = 1;
			pos = 2;
			break;
		case CHAR2INT('m', 'i', 'n', 'i'):
			minfl = TRUE;
			amin = InFile.getnum(FALSE, &eol);
			pos = 3;
			break;
		case CHAR2INT('m', 'a', 'x', 'i'):
			maxfl = TRUE;
			amax = InFile.getnum(FALSE, &eol);
			pos = 4;
			break;
		case CHAR2INT('a', 'm', 'p', 'l'):
			eol = TRUE;
			break;
		case CHAR2INT('i', 'n', 't', 'e'):
			mp = 2;
			eol = TRUE;
			break;
		}
		if (!eol)
			word = InFile.getwrd(FALSE, &eol, &half_codes[pos], length - pos);
	};

	if (RecFile.Open("recfil") != 0)
	{
		fprintf(output, "\n **** unable to open recfil");
		fprintf(output, "\n **** SKUNK execution aborted\n");
		return;
	}

	if (RecFile.GetNelem(&nelem) != 0)
	{
		;
	}

	if (RecFile.GetProjName(prjnam) != 0)
	{
		;
	}

	area = nelem * nelem;

	InFile.listit(flag);

	// if phantom or differential read phantom
	if (phan || (type != 0))
	{

		Phantom = new REAL[area];

		if (RecFile.ReadPhan(phnnam, Phantom) != 0)
		{
			fprintf(output, "\n **** phantom not present");
			fprintf(output, "\n **** SKUNK execution aborted\n");
			delete[] Phantom;  // bug 92 - Lajos - 03/02/2005
			return;
		}
	}

	if (phan)
	{
		fprintf(output, "\n          skunk display of phantom\n");

		fprintf(output, "\n          phantom name:    %s", phnnam);

		// bug 124 - restored flag checks - swr - 7/5/05
		if (!minfl)
		{
			minfl = TRUE;
			amin = Phantom[0];
			for (i = 1; i < area; i++)
			{
				if (Phantom[i] < amin)
					amin = Phantom[i];
			}
		}

		if (!maxfl)
		{
			maxfl = TRUE;
			amax = Phantom[0];
			for (i = 1; i < area; i++)
			{
				if (Phantom[i] > amax)
					amax = Phantom[i];
			}
		}

		if (amax <= amin)
		{
			fprintf(output,
					"\n **** maximal density is less or equal to minimal density");
			fprintf(output, "\n **** SKUNK execution aborted\n");
			delete[] Phantom;  // bug 92 - Lajos - 03/02/2005
			return;
		}

		sprintf(coment,
				"This is phantom was created by snark14: Phantom Name %s",
				phnnam);

		char PhantomName[41];

		strncpy(PhantomName, phnnam, 40);
		PhantomName[40] = 0; // terminate string

		char PhantomName2[41];

		// trim whitespaces
		strtrim(PhantomName2, PhantomName);

		sprintf(FileName, "%s.pgm", PhantomName2);

		// bug 169 - swr - 9/25/05
		for (i = 0; i < 100; ++i)
		{
			if (FileName[i] == ' ')
				FileName[i] = '_';
		}

		SaveImage(Phantom, amax, amin, nelem, nelem, nran, mp, FileName,
				coment);
	}

	Recon = new REAL[area];

	while (RecFile.ReadRec(recnam, algn, &count, &iter, Recon) == 0)
	{

		// continue if reconstruction not selected
		if (flag[iter - 1] == 0)
		{
			continue;
		}

		if (type == 0)
		{
			fprintf(output,
					"\n          skunk display of reconstruction using %s at iter %4i\n",
					algn, count);
		}
		else
		{
			fprintf(output,
					"\n          skunk difference picture using %s at iter %4i\n",
					algn, count);

			for (i = 0; i < area; i++)
			{
				Recon[i] = (REAL) fabs(Phantom[i] - Recon[i]);
			}
		}

		fprintf(output, "\n          projection data: %s", prjnam);

		fprintf(output, "\n          execution name:  %s", recnam);

		// bug 124 - restored flag checks - swr - 7/5/05
		if (!minfl)
		{
			minfl = TRUE;
			amin = Recon[0];
			for (i = 1; i < area; i++)
			{
				if (Recon[i] < amin)
					amin = Recon[i];
			}
		}

		if (!maxfl)
		{
			maxfl = TRUE;
			amax = Recon[0];
			for (i = 1; i < area; i++)
			{
				if (Recon[i] > amax)
					amax = Recon[i];
			}
		}

		if (amax <= amin)
		{
			fprintf(output,
					"\n **** maximal density is less or equal to minimal density");
			fprintf(output, "\n **** SKUNK execution aborted\n");
			break;
		}

		char ProjectionName[13];

		strncpy(ProjectionName, prjnam, 12);
		ProjectionName[12] = 0; // terminate string

		char ProjectionName2[13];

		// trim whitespaces
		strtrim(ProjectionName2, ProjectionName);

		char ReconstructionName[13];

		strncpy(ReconstructionName, recnam, 12);
		ReconstructionName[12] = 0; // terminate string

		char ReconstructionName2[13];

		// trim whitespaces
		strtrim(ReconstructionName2, ReconstructionName);

		sprintf(FileName, "%s_%s_%4s_%04u_%c_%c.pgm", ProjectionName2,
				ReconstructionName2, algn, count, (type != 0) ? 'd' : 'r',
				(mp == 2) ? 'i' : 'a');
		sprintf(coment,
				"This is reconstruction was created by snark14: Algorithm Name %4s Iteration %d",
				algn, count);

		// bug 169 - swr - 9/25/05
		for (i = 0; i < 100; ++i)
		{
			if (FileName[i] == ' ')
				FileName[i] = '_';
		}

		SaveImage(Recon, amax, amin, nelem, nelem, nran, mp, FileName, coment);
	}

	delete[] Recon;  // bug 92 - Lajos - 03/02/2005

	if (phan || (type != 0))
	{
		delete[] Phantom;  // bug 92 - Lajos - 03/02/2005
	}

	RecFile.Close();

	fprintf(output, "\n");
	return;
}
