/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/posit.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 POSIT IS USED TO DETERMINE THE EQUATION OF A RAY.  ITS INPUT
 CONSISTS OF THE NUMBER OF THE RAY AND THE SINE AND COSINE OF
 THE PROJECTION ANGLE.  IT RETURNS THE COORDINATES OF A POINT
 ON THE RAY, (AX,AY), AND THE SLOPE OF THE RAY, (MX,MY).  THE
 ROUTINE ACCESSES GEOMETRY INFORMATION CONTAINED IN /GEOM/ AND
 ASSUMES THAT GENPHI HAS BEEN CALLED TO COMPUTE THE SINE AND COSINE
 TABLES FOR THE DIVERGENT GEOMETRY CASE.
 */

#include <cstdlib>
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "uiod.h"

#include "anglst.h"
#include "posit.h"

void posit(INTEGER np, INTEGER nr, REAL* ax, REAL* ay, REAL* mx, REAL* my)
{
	REAL costh;
	REAL sinth;

	REAL aa, rwidth;

	INTEGER ind;

// CHECK VALIDITY OF PARAMETERS NP AND NR

	if ((np < 0) || (np >= GeoPar.prjnum))
	{
		fprintf(output, "\n **** np = %d is out of range (0..%d)", np,
				GeoPar.prjnum - 1);
		fprintf(output, "\n **** program aborted\n");
		exit(777);
	}

	if ((nr < 0) || (nr >= GeoPar.nrays))
	{
		fprintf(output, "\n **** nr = %d is out of range (0..%d)", nr,
				GeoPar.nrays - 1);
		fprintf(output, "\n **** program aborted\n");
		exit(777);
	}

	// INITIALIZATION
	sinth = Anglst.bsin[np];
	costh = Anglst.bcos[np];

	ind = nr - GeoPar.midray;

	// BREAK HERE FOR DIVERGENT OR PARALLEL GEOMETRY

	if (!GeoPar.par)
	{

		// DIVERGENT CALCULATIONS FOLLOW
		// RETURN THE POSITION OF THE SOURCE AS THE POINT ON THE RAY.
		// THE ANGLE OF THE RAY IS THE DIFFERENCE OF THE PROJECTION ANGLE
		// AND THE ANGLE THE RAY MAKES WITH THE CENTER RAY.

		*ax = GeoPar.radius * costh;
		*ay = GeoPar.radius * sinth;

		if (ind < 0)
		{
			*mx = costh * Anglst.cphi[(-ind) - 1]
					- sinth * Anglst.sphi[(-ind) - 1];
			*my = sinth * Anglst.cphi[(-ind) - 1]
					+ costh * Anglst.sphi[(-ind) - 1];
		}
		else if (ind == 0)
		{
			*mx = costh;
			*my = sinth;
		}
		else if (ind > 0)
		{
			*mx = costh * Anglst.cphi[ind - 1] + sinth * Anglst.sphi[ind - 1];
			*my = sinth * Anglst.cphi[ind - 1] - costh * Anglst.sphi[ind - 1];
		}
	}
	else
	{

		// PARALLEL CALCULATIONS FOLLOW
		// COMPUTE THE INTERSECTION OF THE RAY WITH A LINE PERPENDICULAR TO
		// IT AND PASSING THROUGH THE ORIGIN.  THE ANGLE IS JUST THE
		// PROJECTION ANGLE.

		if (GeoPar.uni)
			rwidth = GeoPar.pinc;
		if (GeoPar.vri)
			rwidth = MAX0((REAL) fabs(sinth), (REAL) fabs(costh))
					* GeoPar.pinc;
		aa = rwidth * ind;
		*ax = -sinth * aa;
		*ay = costh * aa;
		*mx = costh;
		*my = sinth;
	}


	if (trace > 7)
	{
		fprintf(output, "\n          posit   ax = %25.20f ay = %25.20f", *ax,
				*ay);
		fprintf(output, "\n                  mx = %25.20f my = %25.20f", *mx,
				*my);
	}
	return;
}
