/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_prsemi.cpp $
 $LastChangedRevision: 89 $
 $Date: 2014-07-02 17:24:53 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 THIS ROUTINE CALCULATES THE PARAMETERS FOR THE RICHARDSON METHOD
 AND THE SEMI-ITERATIVE METHOD
 HERE

 Z = 4*((LAMBDA + ESTMIN)/(LAMBDA - ESTMIN))**2

 ETA = 2/(LAMBDA + ESTMIN)

 ORDER IS THE VECTOR CONTAINING THE ORDER FOR PICKING THE DELTA
 FOR RICHARDSON METHOD
 DEL IS THE VECTOR OF DELTA FOR THE RICHARDSON METHOD
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "consts.h"
#include "uiod.h"

#include "quad.h"

void quad_class::prsemi(REAL estmin, REAL lambda, BOOLEAN* alg, REAL* delta)
{
	INTEGER i;
	INTEGER j;
	INTEGER k;
	INTEGER n;
	INTEGER m;
	INTEGER ll;
	INTEGER* ic;
	INTEGER iorder;
	REAL pdm;
	REAL botm;
	//REAL* del;   //wei,3/2005  bug 174, wei, 10/2005

	z = lambda - estmin;
	eta = lambda + estmin;  //bug in Fortran. undefined eta hstau
	if (fabs(z) <= Consts.zero)
	{
		*alg = TRUE;
		fprintf(output,
				"\n\n       ***difference of eigenvalues is less than ZERO, program stops***");
		return;
	}

	// RICHARDSON METHOD
	if (!((period > 0) && (period <= 32)))
	{
		fprintf(output,
				"\n\n       ***period out of range (1-32), program stops***");
		*alg = TRUE;
		return;
	}

	if (period == 1)
	{
		// RICHARDSON IS A STATIONARY METHOD
		*delta = (REAL) 2.0 / eta;
		fprintf(output,
				"\n\n       with period=1 this is stationary method with delta = %15.6e",
				*delta);
		bopt = 1;
		return;
	}

	if ((dopt - 2) >= 0)
	{
		if ((dopt - 2) > 0)
		{
			// LEBEDEV -FINOGENOV ORDERING

			// CHECK TO SEE THAT PERIOD IS A POWER OF 2

			m = period;
			for (i = 1; i < 6; i++)
			{   //corr. hstau
				if (m != (m / 2) * 2)
					goto L5;
				m = m / 2;
				if (m == 1)
					goto L6;
			}

			L5: fprintf(output,
					"\n\n      ***for lebedev-finogenov ordering period must");
			fprintf(output, "\n      be a power of 2, program stops***");
			*alg = TRUE;
			return;

			L6: order[0] = 1;
			k = 1;
			for (j = 1; j <= i; j++)
			{  // index j is not explicit in the loops.hstau
				ll = 2 * k + 1;
				//for(m = 0; m < k; m++) {
				for (m = 1; m <= k; m++)
				{
					n = k - m;
					order[2 * n + 1] = ll - order[n];   // corr.hstau
					iorder = 2 * n;  //corr. hstau
					order[iorder] = order[n];
				}
				k = 2 * k;
			}

			fprintf(output, "\n      lebedev-finogenov ordering");
			for (i = 0; i < period; i++)
			{
				if ((i % 32) == 0)
				{
					fprintf(output, "\n      ");
				}
				fprintf(output, "%3i ", order[i]);
			}
		}
		else
		{
			// USER SUPPLIED ORDER
			// CHECK USER ORDER

			// ic = new INTEGER[period];
			ic = new INTEGER[period + 1]; // corr. hstau

			//for(i = 0; i < period; i++) {
			for (i = 1; i <= period; i++)
			{  // corr.hstau
				///ici = ic + i;
				ic[i] = 0;
			}

			for (i = 0; i < period; i++)
			{
				///iorder = order + i;
				j = order[i];
				ic[j]++;
				if (!((order[i] > 0) && (order[i] <= period)))
				{
					fprintf(output,
							"\n      ***%2i th order out of range, program stops***",
							i);
					*alg = TRUE;
				}
			}

			for (i = 1; i <= period; i++)
			{  // corr.hstau
				///ici = ic + i;
				if (ic[i] != 1)
				{
					fprintf(output,
							"\n       warning*** the %3ith delta is used %3i times",
							i, ic[i]);
				}
			}

			delete[] ic;  // bug 92 - Lajos - 03/02/2005

			fprintf(output, "\n\n      user supplied ordering");
			for (i = 0; i < period; i++)
			{
				if ((i % 32) == 0)
				{
					fprintf(output, "\n      ");
				}
				fprintf(output, "%3i ", order[i]);
			}
		}
	}
	// NATURAL ORDER
	else
	{
		for (i = 0; i < period; i++)
		{
			//order[i] = i;
			order[i] = i + 1; //corr.hstau
		}

		fprintf(output, "\n\n      natural ordering");
		// COMPUTE DELTA
	}
	del = new REAL[period];

	pdm = Consts.pi / ((REAL) 2.0 * ((REAL) (period))); //pi? hstau

	for (i = 0; i < period; i++)
	{
		botm = eta - z * (REAL) cos((2.0 * ((REAL) (i)) - 1.0) * pdm);
		if (fabs(botm) <= Consts.zero)
		{
			*alg = TRUE;
			fprintf(output,
					"\n\n      ***cannot compute delta, zero denominator, program stops");
			// if(del != NULL) delete[] del;  //wei, 3/2005  bug 174, wei, 10/2005
			return;
		}
		del[i] = (REAL) 2.0 / botm;

	}
	//if(del != NULL) delete[] del;  //wei, 3/2005 bug 174, wei, 10/2005
	return;
}
