/** @file line_real_t.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include <exception>
#include <stdexcept>

#include <qstring.h>

#include "sd_line_t.hpp"
#include "line_real_t.hpp"
#include "line_real_set_t.hpp"

#include <iostream>

class line_real_set_t;

line_real_t::line_real_t(QString name) {
  plotname = name;
  mySetP = 0;
}

line_real_t::line_real_t(QString name, int fromX, int toX, double atY) { // good for phantom
  plotname = name;
  mySetP = 0;
  for(int x = fromX; x <= toX; x++) {
    add(x,atY);
  }
}

bool line_real_t::empty() { return data.empty(); }

void line_real_t::add(int x, double y) {
  point_real_t p = {x, y};
  data.push_back(p);
} // --line_real_t::add()

void line_real_t::clear() { data.clear(); }

// these 4 must wait until the end of line_real_set_t def

int line_real_t::getMinX() {
  if(0 == mySetP)
    throw std::logic_error("line_real's set not set!");
  return mySetP->getMinX();
}

int line_real_t::getMaxX() {
  if(0 == mySetP)
    throw std::logic_error("line_real's set not set!");
  return mySetP->getMaxX();
}

double line_real_t::getMinY() {
  if(0 == mySetP)
    throw std::logic_error("line_real's set not set!");
  return mySetP->getMinY();
}

double line_real_t::getMaxY() {
  if(0 == mySetP)
    throw std::logic_error("line_real's set not set!");
  return mySetP->getMaxY();
}

QString line_real_t::getName() { return plotname; }

void line_real_t::setset(line_real_set_t* sp) {  mySetP = sp;  }

/** debug routine -- caller is responsible */
void line_real_t::show() {
  for(data_real_t::const_iterator it = data.begin(); it != data.end(); it++) {
    std::cerr << "(" << (it->x) << "," << (it->y) << ")" << " ";
  }
  std::cerr << " ##### ";
}
