/** @file plot_t.hpp
    @author Deniz
    @description class plot_t is defined here
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/
#ifndef _PLOT_T
#define _PLOT_T

#include "sd_line_t.hpp"
#include "line_window_onevar_t.hpp"

class line_real_set_t;
class QString;
class QPainter;

typedef std::list<line_window_onevar_t> data_manyexecs_t;

/** ultimate window-coord class: holds SEVERAL vars' info and can plot them in different colors */
class plot_t
{
  int myWidth, myHeight;
  data_manyexecs_t data;
  QPainter* myPainterP;
  int rMinX, rMaxX;
  double rMinY, rMaxY;
  int xr_range;
  double yr_range;
  int xwmin, xwmax, ywmin, ywmax, xwrange, ywrange;
  double xscalar, yscalar;
  QString plotName, XAxisName, YAxisName;
public:
  /** plot_t ctor/3
   *  prepare a plot based on line_real_set_t and WIDTH and HEIGHT args 
   *  name can be set in setName()
   */
  plot_t(line_real_set_t& LRS, int WIDTH=sd_line_t::WIDTH, int HEIGHT=sd_line_t::HEIGHT);

  /** give a name to this plot_t object */
  void setName(QString& inName);

  void setXAxisName(QString& inNameX);

  void setYAxisName(QString& inNameY);

  /** calculate and make appropriate and human-oriented range increment for Y Axis:
   *  1, 2, or 5 times a power of 10
   *  such there are usually about 10 to 20 marks on the axis
   *  this calibration can be fudged to about halve or double the number of notches
   */
  void makeNotchesOnYAxis(QPainter& P);
  
  /** just like the above, slightly trickier since we only want integers
   */
  void makeNotchesOnXAxis(QPainter& P);

  /** plot the plot_t object */
  void plot(QPainter& P);
  void plotGray(QPainter& P);
}; // -- class plot_t
#endif
