/** @file basis.cpp
    @package snark14Input
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

/* basis.cpp
   This file is part of the snark14Input package
   Written by Bruno M. Carvalho and Deniz Sarioz
   licensed under QPLv1.0 which should accompany this distribution in the file 'QPL'
*/ 

#include "basis.h"
#include "misc.h"
#include "variables.h"
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>

/** 
 *  Constructs a basiswindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
@param void
@author Bruno M. Carvalho
@version 1.0 */
basiswindow::basiswindow( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "basiswindow" );
    resize( 400, 300 );
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 400, 300 ) );
    setMaximumSize( QSize( 400, 300 ) );
    setCaption( tr( "Basis Window" ) );

    basislabel = new QLabel( this, "basislabel" );
    basislabel->setGeometry( QRect( 10, 10, 120, 30 ) ); 
    basislabel->setText( tr( "BASIS" ) );

    basisbuttongroup = new QButtonGroup( this, "basisbuttongroup" );
    basisbuttongroup->setGeometry( QRect( 20, 40, 360, 170 ) ); 
    basisbuttongroup->setTitle( tr( "basis function" ) );
    basisbuttongroup->setAlignment( int( QButtonGroup::AlignHCenter ) );

    pixelbutton = new QRadioButton( basisbuttongroup, "pixelbutton" );
    pixelbutton->setGeometry( QRect( 10, 20, 120, 30 ) ); 
    pixelbutton->setText( tr( "PIXEL" ) ); 
    pixelbutton->setChecked(true);
    QObject::connect(pixelbutton,SIGNAL(clicked()),this,SLOT(deactivateBlobFields()));

    blobbutton = new QRadioButton( basisbuttongroup, "blobbutton" );
    blobbutton->setGeometry( QRect( 10, 70, 80, 30 ) ); 
    blobbutton->setText( tr( "BLOB" ) ); 
    QObject::connect(blobbutton,SIGNAL(clicked()),this,SLOT(activateBlobFields()));

    // "paragraph" ++ by deniz
    useblobdefaultsbox = new QCheckBox( basisbuttongroup, "useblobdefaultsbox" );
    useblobdefaultsbox->setGeometry( QRect( 10, 115, 135, 30 ) );
    useblobdefaultsbox->setText( tr( "use blob defaults" ) );
    useblobdefaultsbox->setEnabled(FALSE);
    //    useblobdefaultsbox->setChecked(false);
    QObject::connect(useblobdefaultsbox,SIGNAL(clicked()),this,SLOT(deactivateBlobFields()));

    supportlabel = new QLabel( basisbuttongroup, "supportlabel" );
    supportlabel->setGeometry( QRect( 105, 50, 60, 20 ) ); 
    supportlabel->setText( tr( "support" ) ); 
    supportlabel->setEnabled(FALSE);

    support = new QLineEdit( basisbuttongroup, "support" );
    support->setGeometry( QRect( 90, 70, 70, 30 ) ); 
    support->setEnabled(FALSE);

    shapelabel = new QLabel( basisbuttongroup, "shapelabel" );
    shapelabel->setEnabled( FALSE );
    shapelabel->setGeometry( QRect( 185, 50, 60, 20 ) ); 
    shapelabel->setText( tr( "shape" ) );

    shape = new QLineEdit( basisbuttongroup, "shape" );
    shape->setGeometry( QRect(170, 70, 70, 30 ) ); 
    shape->setEnabled(FALSE);

    deltalabel = new QLabel( basisbuttongroup, "deltalabel" );
    deltalabel->setEnabled( FALSE );
    deltalabel->setGeometry( QRect( 265, 50, 60, 20 ) ); 
    deltalabel->setText( tr( "delta" ) );

    delta = new QLineEdit( basisbuttongroup, "delta" );
    delta->setGeometry( QRect( 250, 70, 70, 30 ) ); 
    delta->setEnabled(FALSE);

    Line9 = new QFrame( this, "Line9" );
    Line9->setGeometry( QRect( -10, 215, 420, 16 ) ); 
    Line9->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    okbutton = new QPushButton( this, "okbutton" );
    okbutton->setGeometry( QRect( 50, 230, 100, 40 ) ); 
    okbutton->setText( tr( "OK" ) );
    QObject::connect(okbutton,SIGNAL(clicked()),this,SLOT(checkValues()));

    cancelbutton = new QPushButton( this, "cancelbutton" );
    cancelbutton->setGeometry( QRect( 250, 230, 100, 40 ) ); 
    cancelbutton->setText( tr( "Cancel" ) );
    QObject::connect(cancelbutton,SIGNAL(clicked()),this,SLOT(reject()));
}

/**  
 *  Destroys the object and frees any allocated resources
 */
basiswindow::~basiswindow()
{
    // no need to delete child widgets, Qt does it all for us
}

/** Activates blob subfields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void basiswindow::activateBlobFields()
{
  if(blobbutton->isChecked()) {
    useblobdefaultsbox->setEnabled(TRUE);
    useblobdefaultsbox->setChecked(false);
    support->setEnabled(TRUE);
    supportlabel->setEnabled(TRUE);
    shape->setEnabled(TRUE);
    shapelabel->setEnabled(TRUE);
    delta->setEnabled(TRUE);
    deltalabel->setEnabled(TRUE);
  }
  else {
    useblobdefaultsbox->setEnabled(FALSE);
    support->setEnabled(FALSE);
    supportlabel->setEnabled(FALSE);
    shape->setEnabled(FALSE);
    shapelabel->setEnabled(FALSE);
    delta->setEnabled(FALSE);
    deltalabel->setEnabled(FALSE);
  }
}

/** Dectivates blob subfields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void basiswindow::deactivateBlobFields()
{
  // played with by deniz
  if(pixelbutton->isChecked()) {
    useblobdefaultsbox->setEnabled(FALSE);
    support->setEnabled(FALSE);
    supportlabel->setEnabled(FALSE);
    shape->setEnabled(FALSE);
    shapelabel->setEnabled(FALSE);
    delta->setEnabled(FALSE);
    deltalabel->setEnabled(FALSE);
  } else if(useblobdefaultsbox->isChecked()) {
    support->setEnabled(FALSE);
    supportlabel->setEnabled(FALSE);
    shape->setEnabled(FALSE);
    shapelabel->setEnabled(FALSE);
    delta->setEnabled(FALSE);
    deltalabel->setEnabled(FALSE);
  } else activateBlobFields();
}

/** Returns a string for BASIS input sequence
@param void
@return basis
@author Bruno M. Carvalho
@version 1.0 */
QString basiswindow::getOutput()
{
  QString output;

  output.sprintf("BASIS ");
  if(blobbutton->isChecked()) {
    // ++ by deniz
    if(useblobdefaultsbox->isChecked()) { 
      output.append("BLOBS ");
    } else {
      output.append("BLOB ");
      output+=support->text();
      output.append(" ");
      output+=shape->text();
      output.append(" ");
      output+=delta->text();
      output.append(" ");
    }
  }
  else
    if(pixelbutton->isChecked()) {
      output.append("PIXEL ");
    }
  //output.append("\n");

  return output;
}

void basiswindow::checkValues()
{
  int f=0;
  float su=0,sh=0,d=0;
  
  if((blobbutton->isChecked()) && (!(useblobdefaultsbox->isChecked()))) {
    su=strtof((const char *) (support->text().lower()) ); // lower: bug#148
    if(errnum) 
      f=1; else {
	if(su<=0)
	  f=4;
	sh=strtof((const char *) (shape->text().lower()) );
	if(errnum) 
	  f=2; else {
	    if(sh<0)
	      f=5;
	    d=strtof((const char *) (delta->text().lower()) );
	    if(errnum) 
	      f=3; else {
		if(d<=0)
		  f=6;
	      }
	  }
      }
  }
  switch(f) {
  case 0: accept();
    break;
  case 1: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid support format.");
    break;
  case 2: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid shape format.");
    break;
  case 3: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid delta format.");
    break;
  case 4: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "support must be positive.");
    break;
  case 5: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "shape must be nonnegative.");
    break;
  case 6: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "delta must be positive.");
    break;
  }
}

