/** @file lines.cpp
    @package snark14Input
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "lines.h"
#include "misc.h"
#include "variables.h"

#include <qcheckbox.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>

/** 
 *  Constructs a lineswindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
@param void
@author Bruno M. Carvalho
@version 1.0 */
lineswindow::lineswindow( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "lineswindow" );
    resize( 400, 350 ); 
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 400, 350 ) );
    setMaximumSize( QSize( 400, 350 ) );
    setCaption( tr( "Lines Window" ) );

    lineslabel = new QLabel( this, "lineslabel" );
    lineslabel->setGeometry( QRect( 10, 10, 70, 30 ) ); 
    lineslabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, lineslabel->sizePolicy().hasHeightForWidth() ) );
    lineslabel->setMinimumSize( QSize( 70, 30 ) );
    lineslabel->setMaximumSize( QSize( 70, 30 ) );
    lineslabel->setText( tr( "LINES" ) );

    scalebox = new QCheckBox( this, "scalebox" );
    scalebox->setGeometry( QRect( 30, 60, 80, 30 ) ); 
    scalebox->setText( tr( "SCALE" ) );
    QObject::connect(scalebox,SIGNAL(clicked()),this,SLOT(activateScaleFields()));

    scale = new QLineEdit( this, "scale" );
    scale->setGeometry( QRect( 150, 60, 50, 30 ) ); 
    scale->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, scale->sizePolicy().hasHeightForWidth() ) );
    scale->setEnabled(FALSE);

    scalelabel = new QLabel( this, "scalelabel" );
    scalelabel->setGeometry( QRect( 150, 40, 40, 20 ) ); 
    scalelabel->setText( tr( "scale" ) );
    scalelabel->setEnabled(FALSE);

    columnslabel = new QLabel( this, "columnslabel" );
    columnslabel->setGeometry( QRect( 30, 130, 100, 30 ) ); 
    columnslabel->setText( tr( "COLUMNS" ) );

    col1 = new QLineEdit( this, "col1" );
    col1->setGeometry( QRect( 150, 130, 50, 30 ) ); 
    col1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, col1->sizePolicy().hasHeightForWidth() ) );

    col1label = new QLabel( this, "col1label" );
    col1label->setGeometry( QRect( 160, 110, 30, 20 ) ); 
    col1label->setText( tr( "col1" ) );

    col2 = new QLineEdit( this, "col2" );
    col2->setGeometry( QRect( 210, 130, 50, 30 ) ); 
    col2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, col2->sizePolicy().hasHeightForWidth() ) );

    col2label = new QLabel( this, "col2label" );
    col2label->setGeometry( QRect( 220, 110, 30, 20 ) ); 
    col2label->setText( tr( "col2" ) );

    col3 = new QLineEdit( this, "col3" );
    col3->setGeometry( QRect( 270, 130, 50, 30 ) ); 
    col3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, col3->sizePolicy().hasHeightForWidth() ) );

    col3label = new QLabel( this, "col3label" );
    col3label->setGeometry( QRect( 280, 110, 30, 20 ) ); 
    col3label->setText( tr( "col3" ) );

    col4 = new QLineEdit( this, "col4" );
    col4->setGeometry( QRect( 330, 130, 50, 30 ) ); 
    col4->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, col4->sizePolicy().hasHeightForWidth() ) );

    col4label = new QLabel( this, "col4label" );
    col4label->setGeometry( QRect( 340, 110, 30, 20 ) ); 
    col4label->setText( tr( "col4" ) );

    iterationflaglabel = new QLabel( this, "iterationflaglabel" );
    iterationflaglabel->setGeometry( QRect( 135, 190, 130, 20 ) ); 
    iterationflaglabel->setText( tr( "iteration_flag_line" ) );

    iterationflagline = new QLineEdit( this, "iterationflagline" );
    iterationflagline->setGeometry( QRect( 50, 210, 310, 30 ) ); 

    okbutton = new QPushButton( this, "okbutton" );
    okbutton->setGeometry( QRect( 50, 290, 100, 40 ) ); 
    okbutton->setText( tr( "OK" ) );
    QObject::connect(okbutton,SIGNAL(clicked()),this,SLOT(checkValues()));

    cancelbutton = new QPushButton( this, "cancelbutton" );
    cancelbutton->setGeometry( QRect( 250, 290, 100, 40 ) ); 
    cancelbutton->setText( tr( "Cancel" ) );
    QObject::connect(cancelbutton,SIGNAL(clicked()),this,SLOT(reject()));

    Line7 = new QFrame( this, "Line7" );
    Line7->setGeometry( QRect( 0, 275, 400, 1 ) ); 
    Line7->setFrameStyle( QFrame::HLine | QFrame::Sunken ); 
}

/**  
 *  Destroys the object and frees any allocated resources
 */
lineswindow::~lineswindow()
{
    // no need to delete child widgets, Qt does it all for us
}

/** Activates scale subfields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void lineswindow::activateScaleFields()
{
  if(scalebox->isChecked()) {
    scale->setEnabled(TRUE);
    scalelabel->setEnabled(TRUE);
  }
  else {
    scale->setEnabled(FALSE);
    scalelabel->setEnabled(FALSE);
  }
}

/** Returns a string for LINES input sequence
@param void
@return lines
@author Bruno M. Carvalho
@version 1.0 */
QString lineswindow::getOutput()
{
  QString output;

 
  output.sprintf("LINES ");
  if(scalebox->isChecked()) {
    output.append("SCALE ");
    output+=scale->text();
    output.append(" ");
  }
  output.append("COLUMNS ");
  output+=col1->text();
  output.append(" ");
  output+=col2->text();
  output.append(" ");
  output+=col3->text();
  output.append(" ");
  output+=col4->text();
  output.append(" ");
  output.append("\n");
  output+=iterationflagline->text();
  //output.append("\n");

  return output;
}

void lineswindow::checkValues()
{
  int f=0,c1,c2,c3,c4;
  float s=0;
  
  if(scalebox->isChecked()) {
    s=strtof((const char *)scale->text());
    if(errnum) 
      f=1;
    if(s<=0)
      f=2;
  }
  c1=strtoi((const char *)col1->text());
  if(errnum) 
    f=3;
  c2=strtoi((const char *)col2->text());
  if(errnum==1) 
    f=4;
  c3=strtoi((const char *)col3->text());
  if(errnum==1) 
    f=5;
  c4=strtoi((const char *)col4->text());
  if(errnum==1) 
    f=6;
  if(c1<0)
    f=7;
  if(c2<0)
    f=8;
  if(c3<0)
    f=9;
  if(c4<0)
    f=10;
  if(!isintspace(iterationflagline->text()))
    f=11;
  switch(f) {
  case 0: accept();
    break;
  case 1: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid scale format");
          scale->setFocus();
    break;
  case 2: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "scale should be positive");
          scale->setFocus();
    break;
  case 3: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid col1 format");
          col1->setFocus();
    break;
  case 4: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid col2 format");
          col2->setFocus();
    break;
  case 5: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid col3 format");
          col3->setFocus();
    break;
  case 6: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid col4 format");
          col4->setFocus();
    break;
  case 7: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "col1 must be positive");
          col1->setFocus();
    break;
  case 8: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "col2 must be positive or empty");
          col2->setFocus();
    break;
  case 9: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "col3 must be positive or empty");
          col3->setFocus();
    break;
  case 10: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "col4 must be positive or empty");
          col4->setFocus();
    break;
  case 11: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "iteration_flag_line must contain\n"
                                   "only 0-9 and spaces and must\n"
                                   "be non-empty");
          iterationflagline->setFocus();
    break;
  }
}
